/*! nouislider - 9.2.0 - 2017-01-11 10:35:34 */

(function (factory) {

    if ( typeof define === 'function' && define.amd ) {

        // AMD. Register as an anonymous module.
        define([], factory);

    } else if ( typeof exports === 'object' ) {

        // Node/CommonJS
        module.exports = factory();

    } else {

        // Browser globals
        window.noUiSlider = factory();
    }

}(function( ){

    'use strict';

    var VERSION = '9.2.0';


    // Creates a node, adds it to target, returns the new node.
    function addNodeTo ( target, className ) {
        var div = document.createElement('div');
        addClass(div, className);
        target.appendChild(div);
        return div;
    }

    // Removes duplicates from an array.
    function unique ( array ) {
        return array.filter(function(a){
            return !this[a] ? this[a] = true : false;
        }, {});
    }

    // Round a value to the closest 'to'.
    function closest ( value, to ) {
        return Math.round(value / to) * to;
    }

    // Current position of an element relative to the document.
    function offset ( elem, orientation ) {

        var rect = elem.getBoundingClientRect(),
            doc = elem.ownerDocument,
            docElem = doc.documentElement,
            pageOffset = getPageOffset();

        // getBoundingClientRect contains left scroll in Chrome on Android.
        // I haven't found a feature detection that proves this. Worst case
        // scenario on mis-match: the 'tap' feature on horizontal sliders breaks.
        if ( /webkit.*Chrome.*Mobile/i.test(navigator.userAgent) ) {
            pageOffset.x = 0;
        }

        return orientation ? (rect.top + pageOffset.y - docElem.clientTop) : (rect.left + pageOffset.x - docElem.clientLeft);
    }

    // Checks whether a value is numerical.
    function isNumeric ( a ) {
        return typeof a === 'number' && !isNaN( a ) && isFinite( a );
    }

    // Sets a class and removes it after [duration] ms.
    function addClassFor ( element, className, duration ) {
        if (duration > 0) {
            addClass(element, className);
            setTimeout(function(){
                removeClass(element, className);
            }, duration);
        }
    }

    // Limits a value to 0 - 100
    function limit ( a ) {
        return Math.max(Math.min(a, 100), 0);
    }

    // Wraps a variable as an array, if it isn't one yet.
    // Note that an input array is returned by reference!
    function asArray ( a ) {
        return Array.isArray(a) ? a : [a];
    }

    // Counts decimals
    function countDecimals ( numStr ) {
        numStr = String(numStr);
        var pieces = numStr.split(".");
        return pieces.length > 1 ? pieces[1].length : 0;
    }

    // http://youmightnotneedjquery.com/#add_class
    function addClass ( el, className ) {
        if ( el.classList ) {
            el.classList.add(className);
        } else {
            el.className += ' ' + className;
        }
    }

    // http://youmightnotneedjquery.com/#remove_class
    function removeClass ( el, className ) {
        if ( el.classList ) {
            el.classList.remove(className);
        } else {
            el.className = el.className.replace(new RegExp('(^|\\b)' + className.split(' ').join('|') + '(\\b|$)', 'gi'), ' ');
        }
    }

    // https://plainjs.com/javascript/attributes/adding-removing-and-testing-for-classes-9/
    function hasClass ( el, className ) {
        return el.classList ? el.classList.contains(className) : new RegExp('\\b' + className + '\\b').test(el.className);
    }

    // https://developer.mozilla.org/en-US/docs/Web/API/Window/scrollY#Notes
    function getPageOffset ( ) {

        var supportPageOffset = window.pageXOffset !== undefined,
            isCSS1Compat = ((document.compatMode || "") === "CSS1Compat"),
            x = supportPageOffset ? window.pageXOffset : isCSS1Compat ? document.documentElement.scrollLeft : document.body.scrollLeft,
            y = supportPageOffset ? window.pageYOffset : isCSS1Compat ? document.documentElement.scrollTop : document.body.scrollTop;

        return {
            x: x,
            y: y
        };
    }

    // we provide a function to compute constants instead
    // of accessing window.* as soon as the module needs it
    // so that we do not compute anything if not needed
    function getActions ( ) {

        // Determine the events to bind. IE11 implements pointerEvents without
        // a prefix, which breaks compatibility with the IE10 implementation.
        return window.navigator.pointerEnabled ? {
                start: 'pointerdown',
                move: 'pointermove',
                end: 'pointerup'
            } : window.navigator.msPointerEnabled ? {
                    start: 'MSPointerDown',
                    move: 'MSPointerMove',
                    end: 'MSPointerUp'
                } : {
                    start: 'mousedown touchstart',
                    move: 'mousemove touchmove',
                    end: 'mouseup touchend'
                };
    }


// Value calculation

    // Determine the size of a sub-range in relation to a full range.
    function subRangeRatio ( pa, pb ) {
        return (100 / (pb - pa));
    }

    // (percentage) How many percent is this value of this range?
    function fromPercentage ( range, value ) {
        return (value * 100) / ( range[1] - range[0] );
    }

    // (percentage) Where is this value on this range?
    function toPercentage ( range, value ) {
        return fromPercentage( range, range[0] < 0 ?
            value + Math.abs(range[0]) :
            value - range[0] );
    }

    // (value) How much is this percentage on this range?
    function isPercentage ( range, value ) {
        return ((value * ( range[1] - range[0] )) / 100) + range[0];
    }


// Range conversion

    function getJ ( value, arr ) {

        var j = 1;

        while ( value >= arr[j] ){
            j += 1;
        }

        return j;
    }

    // (percentage) Input a value, find where, on a scale of 0-100, it applies.
    function toStepping ( xVal, xPct, value ) {

        if ( value >= xVal.slice(-1)[0] ){
            return 100;
        }

        var j = getJ( value, xVal ), va, vb, pa, pb;

        va = xVal[j-1];
        vb = xVal[j];
        pa = xPct[j-1];
        pb = xPct[j];

        return pa + (toPercentage([va, vb], value) / subRangeRatio (pa, pb));
    }

    // (value) Input a percentage, find where it is on the specified range.
    function fromStepping ( xVal, xPct, value ) {

        // There is no range group that fits 100
        if ( value >= 100 ){
            return xVal.slice(-1)[0];
        }

        var j = getJ( value, xPct ), va, vb, pa, pb;

        va = xVal[j-1];
        vb = xVal[j];
        pa = xPct[j-1];
        pb = xPct[j];

        return isPercentage([va, vb], (value - pa) * subRangeRatio (pa, pb));
    }

    // (percentage) Get the step that applies at a certain value.
    function getStep ( xPct, xSteps, snap, value ) {

        if ( value === 100 ) {
            return value;
        }

        var j = getJ( value, xPct ), a, b;

        // If 'snap' is set, steps are used as fixed points on the slider.
        if ( snap ) {

            a = xPct[j-1];
            b = xPct[j];

            // Find the closest position, a or b.
            if ((value - a) > ((b-a)/2)){
                return b;
            }

            return a;
        }

        if ( !xSteps[j-1] ){
            return value;
        }

        return xPct[j-1] + closest(
                value - xPct[j-1],
                xSteps[j-1]
            );
    }


// Entry parsing

    function handleEntryPoint ( index, value, that ) {

        var percentage;

        // Wrap numerical input in an array.
        if ( typeof value === "number" ) {
            value = [value];
        }

        // Reject any invalid input, by testing whether value is an array.
        if ( Object.prototype.toString.call( value ) !== '[object Array]' ){
            throw new Error("noUiSlider (" + VERSION + "): 'range' contains invalid value.");
        }

        // Covert min/max syntax to 0 and 100.
        if ( index === 'min' ) {
            percentage = 0;
        } else if ( index === 'max' ) {
            percentage = 100;
        } else {
            percentage = parseFloat( index );
        }

        // Check for correct input.
        if ( !isNumeric( percentage ) || !isNumeric( value[0] ) ) {
            throw new Error("noUiSlider (" + VERSION + "): 'range' value isn't numeric.");
        }

        // Store values.
        that.xPct.push( percentage );
        that.xVal.push( value[0] );

        // NaN will evaluate to false too, but to keep
        // logging clear, set step explicitly. Make sure
        // not to override the 'step' setting with false.
        if ( !percentage ) {
            if ( !isNaN( value[1] ) ) {
                that.xSteps[0] = value[1];
            }
        } else {
            that.xSteps.push( isNaN(value[1]) ? false : value[1] );
        }

        that.xHighestCompleteStep.push(0);
    }

    function handleStepPoint ( i, n, that ) {

        // Ignore 'false' stepping.
        if ( !n ) {
            return true;
        }

        // Factor to range ratio
        that.xSteps[i] = fromPercentage([
                that.xVal[i]
                ,that.xVal[i+1]
            ], n) / subRangeRatio (
                that.xPct[i],
                that.xPct[i+1] );

        var totalSteps = (that.xVal[i+1] - that.xVal[i]) / that.xNumSteps[i];
        var highestStep = Math.ceil(Number(totalSteps.toFixed(3)) - 1);
        var step = that.xVal[i] + (that.xNumSteps[i] * highestStep);

        that.xHighestCompleteStep[i] = step;
    }


// Interface

    // The interface to Spectrum handles all direction-based
    // conversions, so the above values are unaware.

    function Spectrum ( entry, snap, direction, singleStep ) {

        this.xPct = [];
        this.xVal = [];
        this.xSteps = [ singleStep || false ];
        this.xNumSteps = [ false ];
        this.xHighestCompleteStep = [];

        this.snap = snap;
        this.direction = direction;

        var index, ordered = [ /* [0, 'min'], [1, '50%'], [2, 'max'] */ ];

        // Map the object keys to an array.
        for ( index in entry ) {
            if ( entry.hasOwnProperty(index) ) {
                ordered.push([entry[index], index]);
            }
        }

        // Sort all entries by value (numeric sort).
        if ( ordered.length && typeof ordered[0][0] === "object" ) {
            ordered.sort(function(a, b) { return a[0][0] - b[0][0]; });
        } else {
            ordered.sort(function(a, b) { return a[0] - b[0]; });
        }


        // Convert all entries to subranges.
        for ( index = 0; index < ordered.length; index++ ) {
            handleEntryPoint(ordered[index][1], ordered[index][0], this);
        }

        // Store the actual step values.
        // xSteps is sorted in the same order as xPct and xVal.
        this.xNumSteps = this.xSteps.slice(0);

        // Convert all numeric steps to the percentage of the subrange they represent.
        for ( index = 0; index < this.xNumSteps.length; index++ ) {
            handleStepPoint(index, this.xNumSteps[index], this);
        }
    }

    Spectrum.prototype.getMargin = function ( value ) {

        var step = this.xNumSteps[0];

        if ( step && ((value / step) % 1) !== 0 ) {
            throw new Error("noUiSlider (" + VERSION + "): 'limit', 'margin' and 'padding' must be divisible by step.");
        }

        return this.xPct.length === 2 ? fromPercentage(this.xVal, value) : false;
    };

    Spectrum.prototype.toStepping = function ( value ) {

        value = toStepping( this.xVal, this.xPct, value );

        return value;
    };

    Spectrum.prototype.fromStepping = function ( value ) {

        return fromStepping( this.xVal, this.xPct, value );
    };

    Spectrum.prototype.getStep = function ( value ) {

        value = getStep(this.xPct, this.xSteps, this.snap, value );

        return value;
    };

    Spectrum.prototype.getNearbySteps = function ( value ) {

        var j = getJ(value, this.xPct);

        return {
            stepBefore: { startValue: this.xVal[j-2], step: this.xNumSteps[j-2], highestStep: this.xHighestCompleteStep[j-2] },
            thisStep: { startValue: this.xVal[j-1], step: this.xNumSteps[j-1], highestStep: this.xHighestCompleteStep[j-1] },
            stepAfter: { startValue: this.xVal[j-0], step: this.xNumSteps[j-0], highestStep: this.xHighestCompleteStep[j-0] }
        };
    };

    Spectrum.prototype.countStepDecimals = function () {
        var stepDecimals = this.xNumSteps.map(countDecimals);
        return Math.max.apply(null, stepDecimals);
    };

    // Outside testing
    Spectrum.prototype.convert = function ( value ) {
        return this.getStep(this.toStepping(value));
    };

	/*	Every input option is tested and parsed. This'll prevent
	 endless validation in internal methods. These tests are
	 structured with an item for every option available. An
	 option can be marked as required by setting the 'r' flag.
	 The testing function is provided with three arguments:
	 - The provided value for the option;
	 - A reference to the options object;
	 - The name for the option;

	 The testing function returns false when an error is detected,
	 or true when everything is OK. It can also modify the option
	 object, to make sure all values can be correctly looped elsewhere. */

    var defaultFormatter = { 'to': function( value ){
        return value !== undefined && value.toFixed(2);
    }, 'from': Number };

    function testStep ( parsed, entry ) {

        if ( !isNumeric( entry ) ) {
            throw new Error("noUiSlider (" + VERSION + "): 'step' is not numeric.");
        }

        // The step option can still be used to set stepping
        // for linear sliders. Overwritten if set in 'range'.
        parsed.singleStep = entry;
    }

    function testRange ( parsed, entry ) {

        // Filter incorrect input.
        if ( typeof entry !== 'object' || Array.isArray(entry) ) {
            throw new Error("noUiSlider (" + VERSION + "): 'range' is not an object.");
        }

        // Catch missing start or end.
        if ( entry.min === undefined || entry.max === undefined ) {
            throw new Error("noUiSlider (" + VERSION + "): Missing 'min' or 'max' in 'range'.");
        }

        // Catch equal start or end.
        if ( entry.min === entry.max ) {
            //throw new Error("noUiSlider (" + VERSION + "): 'range' 'min' and 'max' cannot be equal."); // IT&T
        }

        parsed.spectrum = new Spectrum(entry, parsed.snap, parsed.dir, parsed.singleStep);
    }

    function testStart ( parsed, entry ) {

        entry = asArray(entry);

        // Validate input. Values aren't tested, as the public .val method
        // will always provide a valid location.
        if ( !Array.isArray( entry ) || !entry.length ) {
            throw new Error("noUiSlider (" + VERSION + "): 'start' option is incorrect.");
        }

        // Store the number of handles.
        parsed.handles = entry.length;

        // When the slider is initialized, the .val method will
        // be called with the start options.
        parsed.start = entry;
    }

    function testSnap ( parsed, entry ) {

        // Enforce 100% stepping within subranges.
        parsed.snap = entry;

        if ( typeof entry !== 'boolean' ){
            throw new Error("noUiSlider (" + VERSION + "): 'snap' option must be a boolean.");
        }
    }

    function testAnimate ( parsed, entry ) {

        // Enforce 100% stepping within subranges.
        parsed.animate = entry;

        if ( typeof entry !== 'boolean' ){
            throw new Error("noUiSlider (" + VERSION + "): 'animate' option must be a boolean.");
        }
    }

    function testAnimationDuration ( parsed, entry ) {

        parsed.animationDuration = entry;

        if ( typeof entry !== 'number' ){
            throw new Error("noUiSlider (" + VERSION + "): 'animationDuration' option must be a number.");
        }
    }

    function testConnect ( parsed, entry ) {

        var connect = [false];
        var i;

        // Map legacy options
        if ( entry === 'lower' ) {
            entry = [true, false];
        }

        else if ( entry === 'upper' ) {
            entry = [false, true];
        }

        // Handle boolean options
        if ( entry === true || entry === false ) {

            for ( i = 1; i < parsed.handles; i++ ) {
                connect.push(entry);
            }

            connect.push(false);
        }

        // Reject invalid input
        else if ( !Array.isArray( entry ) || !entry.length || entry.length !== parsed.handles + 1 ) {
            throw new Error("noUiSlider (" + VERSION + "): 'connect' option doesn't match handle count.");
        }

        else {
            connect = entry;
        }

        parsed.connect = connect;
    }

    function testOrientation ( parsed, entry ) {

        // Set orientation to an a numerical value for easy
        // array selection.
        switch ( entry ){
            case 'horizontal':
                parsed.ort = 0;
                break;
            case 'vertical':
                parsed.ort = 1;
                break;
            default:
                throw new Error("noUiSlider (" + VERSION + "): 'orientation' option is invalid.");
        }
    }

    function testMargin ( parsed, entry ) {

        if ( !isNumeric(entry) ){
            throw new Error("noUiSlider (" + VERSION + "): 'margin' option must be numeric.");
        }

        // Issue #582
        if ( entry === 0 ) {
            return;
        }

        parsed.margin = parsed.spectrum.getMargin(entry);

        if ( !parsed.margin ) {
            throw new Error("noUiSlider (" + VERSION + "): 'margin' option is only supported on linear sliders.");
        }
    }

    function testLimit ( parsed, entry ) {

        if ( !isNumeric(entry) ){
            throw new Error("noUiSlider (" + VERSION + "): 'limit' option must be numeric.");
        }

        parsed.limit = parsed.spectrum.getMargin(entry);

        if ( !parsed.limit || parsed.handles < 2 ) {
            throw new Error("noUiSlider (" + VERSION + "): 'limit' option is only supported on linear sliders with 2 or more handles.");
        }
    }

    function testPadding ( parsed, entry ) {

        if ( !isNumeric(entry) ){
            throw new Error("noUiSlider (" + VERSION + "): 'padding' option must be numeric.");
        }

        if ( entry === 0 ) {
            return;
        }

        parsed.padding = parsed.spectrum.getMargin(entry);

        if ( !parsed.padding ) {
            throw new Error("noUiSlider (" + VERSION + "): 'padding' option is only supported on linear sliders.");
        }

        if ( parsed.padding < 0 ) {
            throw new Error("noUiSlider (" + VERSION + "): 'padding' option must be a positive number.");
        }

        if ( parsed.padding >= 50 ) {
            throw new Error("noUiSlider (" + VERSION + "): 'padding' option must be less than half the range.");
        }
    }

    function testDirection ( parsed, entry ) {

        // Set direction as a numerical value for easy parsing.
        // Invert connection for RTL sliders, so that the proper
        // handles get the connect/background classes.
        switch ( entry ) {
            case 'ltr':
                parsed.dir = 0;
                break;
            case 'rtl':
                parsed.dir = 1;
                break;
            default:
                throw new Error("noUiSlider (" + VERSION + "): 'direction' option was not recognized.");
        }
    }

    function testBehaviour ( parsed, entry ) {

        // Make sure the input is a string.
        if ( typeof entry !== 'string' ) {
            throw new Error("noUiSlider (" + VERSION + "): 'behaviour' must be a string containing options.");
        }

        // Check if the string contains any keywords.
        // None are required.
        var tap = entry.indexOf('tap') >= 0;
        var drag = entry.indexOf('drag') >= 0;
        var fixed = entry.indexOf('fixed') >= 0;
        var snap = entry.indexOf('snap') >= 0;
        var hover = entry.indexOf('hover') >= 0;

        if ( fixed ) {

            if ( parsed.handles !== 2 ) {
                throw new Error("noUiSlider (" + VERSION + "): 'fixed' behaviour must be used with 2 handles");
            }

            // Use margin to enforce fixed state
            testMargin(parsed, parsed.start[1] - parsed.start[0]);
        }

        parsed.events = {
            tap: tap || snap,
            drag: drag,
            fixed: fixed,
            snap: snap,
            hover: hover
        };
    }

    function testTooltips ( parsed, entry ) {

        if ( entry === false ) {
            return;
        }

        else if ( entry === true ) {

            parsed.tooltips = [];

            for ( var i = 0; i < parsed.handles; i++ ) {
                parsed.tooltips.push(true);
            }
        }

        else {

            parsed.tooltips = asArray(entry);

            if ( parsed.tooltips.length !== parsed.handles ) {
                throw new Error("noUiSlider (" + VERSION + "): must pass a formatter for all handles.");
            }

            parsed.tooltips.forEach(function(formatter){
                if ( typeof formatter !== 'boolean' && (typeof formatter !== 'object' || typeof formatter.to !== 'function') ) {
                    throw new Error("noUiSlider (" + VERSION + "): 'tooltips' must be passed a formatter or 'false'.");
                }
            });
        }
    }

    function testFormat ( parsed, entry ) {

        parsed.format = entry;

        // Any object with a to and from method is supported.
        if ( typeof entry.to === 'function' && typeof entry.from === 'function' ) {
            return true;
        }

        throw new Error("noUiSlider (" + VERSION + "): 'format' requires 'to' and 'from' methods.");
    }

    function testCssPrefix ( parsed, entry ) {

        if ( entry !== undefined && typeof entry !== 'string' && entry !== false ) {
            throw new Error("noUiSlider (" + VERSION + "): 'cssPrefix' must be a string or `false`.");
        }

        parsed.cssPrefix = entry;
    }

    function testCssClasses ( parsed, entry ) {

        if ( entry !== undefined && typeof entry !== 'object' ) {
            throw new Error("noUiSlider (" + VERSION + "): 'cssClasses' must be an object.");
        }

        if ( typeof parsed.cssPrefix === 'string' ) {
            parsed.cssClasses = {};

            for ( var key in entry ) {
                if ( !entry.hasOwnProperty(key) ) { continue; }

                parsed.cssClasses[key] = parsed.cssPrefix + entry[key];
            }
        } else {
            parsed.cssClasses = entry;
        }
    }

    function testUseRaf ( parsed, entry ) {
        if ( entry === true || entry === false ) {
            parsed.useRequestAnimationFrame = entry;
        } else {
            throw new Error("noUiSlider (" + VERSION + "): 'useRequestAnimationFrame' option should be true (default) or false.");
        }
    }

    // Test all developer settings and parse to assumption-safe values.
    function testOptions ( options ) {

        // To prove a fix for #537, freeze options here.
        // If the object is modified, an error will be thrown.
        // Object.freeze(options);

        var parsed = {
            margin: 0,
            limit: 0,
            padding: 0,
            animate: true,
            animationDuration: 300,
            format: defaultFormatter
        };

        // Tests are executed in the order they are presented here.
        var tests = {
            'step': { r: false, t: testStep },
            'start': { r: true, t: testStart },
            'connect': { r: true, t: testConnect },
            'direction': { r: true, t: testDirection },
            'snap': { r: false, t: testSnap },
            'animate': { r: false, t: testAnimate },
            'animationDuration': { r: false, t: testAnimationDuration },
            'range': { r: true, t: testRange },
            'orientation': { r: false, t: testOrientation },
            'margin': { r: false, t: testMargin },
            'limit': { r: false, t: testLimit },
            'padding': { r: false, t: testPadding },
            'behaviour': { r: true, t: testBehaviour },
            'format': { r: false, t: testFormat },
            'tooltips': { r: false, t: testTooltips },
            'cssPrefix': { r: false, t: testCssPrefix },
            'cssClasses': { r: false, t: testCssClasses },
            'useRequestAnimationFrame': { r: false, t: testUseRaf }
        };

        var defaults = {
            'connect': false,
            'direction': 'ltr',
            'behaviour': 'tap',
            'orientation': 'horizontal',
            'cssPrefix' : 'noUi-',
            'cssClasses': {
                target: 'target',
                base: 'base',
                origin: 'origin',
                handle: 'handle',
                handleLower: 'handle-lower',
                handleUpper: 'handle-upper',
                horizontal: 'horizontal',
                vertical: 'vertical',
                background: 'background',
                connect: 'connect',
                ltr: 'ltr',
                rtl: 'rtl',
                draggable: 'draggable',
                drag: 'state-drag',
                tap: 'state-tap',
                active: 'active',
                tooltip: 'tooltip',
                pips: 'pips',
                pipsHorizontal: 'pips-horizontal',
                pipsVertical: 'pips-vertical',
                marker: 'marker',
                markerHorizontal: 'marker-horizontal',
                markerVertical: 'marker-vertical',
                markerNormal: 'marker-normal',
                markerLarge: 'marker-large',
                markerSub: 'marker-sub',
                value: 'value',
                valueHorizontal: 'value-horizontal',
                valueVertical: 'value-vertical',
                valueNormal: 'value-normal',
                valueLarge: 'value-large',
                valueSub: 'value-sub'
            },
            'useRequestAnimationFrame': true
        };

        // Run all options through a testing mechanism to ensure correct
        // input. It should be noted that options might get modified to
        // be handled properly. E.g. wrapping integers in arrays.
        Object.keys(tests).forEach(function( name ){

            // If the option isn't set, but it is required, throw an error.
            if ( options[name] === undefined && defaults[name] === undefined ) {

                if ( tests[name].r ) {
                    throw new Error("noUiSlider (" + VERSION + "): '" + name + "' is required.");
                }

                return true;
            }

            tests[name].t( parsed, options[name] === undefined ? defaults[name] : options[name] );
        });

        // Forward pips options
        parsed.pips = options.pips;

        var styles = [['left', 'top'], ['right', 'bottom']];

        // Pre-define the styles.
        parsed.style = styles[parsed.dir][parsed.ort];
        parsed.styleOposite = styles[parsed.dir?0:1][parsed.ort];

        return parsed;
    }


    function closure ( target, options, originalOptions ){

        var actions = getActions( );

        // All variables local to 'closure' are prefixed with 'scope_'
        var scope_Target = target;
        var scope_Locations = [];
        var scope_Base;
        var scope_Handles;
        var scope_HandleNumbers = [];
        var scope_ActiveHandle = false;
        var scope_Connects;
        var scope_Spectrum = options.spectrum;
        var scope_Values = [];
        var scope_Events = {};
        var scope_Self;


        // Append a origin to the base
        function addOrigin ( base, handleNumber ) {

            var origin = addNodeTo(base, options.cssClasses.origin);
            var handle = addNodeTo(origin, options.cssClasses.handle);

            handle.setAttribute('data-handle', handleNumber);

            if ( handleNumber === 0 ) {
                addClass(handle, options.cssClasses.handleLower);
            }

            else if ( handleNumber === options.handles - 1 ) {
                addClass(handle, options.cssClasses.handleUpper);
            }

            return origin;
        }

        // Insert nodes for connect elements
        function addConnect ( base, add ) {

            if ( !add ) {
                return false;
            }

            return addNodeTo(base, options.cssClasses.connect);
        }

        // Add handles to the slider base.
        function addElements ( connectOptions, base ) {

            scope_Handles = [];
            scope_Connects = [];

            scope_Connects.push(addConnect(base, connectOptions[0]));

            // [::::O====O====O====]
            // connectOptions = [0, 1, 1, 1]

            for ( var i = 0; i < options.handles; i++ ) {
                // Keep a list of all added handles.
                scope_Handles.push(addOrigin(base, i));
                scope_HandleNumbers[i] = i;
                scope_Connects.push(addConnect(base, connectOptions[i + 1]));
            }
        }

        // Initialize a single slider.
        function addSlider ( target ) {

            // Apply classes and data to the target.
            addClass(target, options.cssClasses.target);

            if ( options.dir === 0 ) {
                addClass(target, options.cssClasses.ltr);
            } else {
                addClass(target, options.cssClasses.rtl);
            }

            if ( options.ort === 0 ) {
                addClass(target, options.cssClasses.horizontal);
            } else {
                addClass(target, options.cssClasses.vertical);
            }

            scope_Base = addNodeTo(target, options.cssClasses.base);
        }


        function addTooltip ( handle, handleNumber ) {

            if ( !options.tooltips[handleNumber] ) {
                return false;
            }

            return addNodeTo(handle.firstChild, options.cssClasses.tooltip);
        }

        // The tooltips option is a shorthand for using the 'update' event.
        function tooltips ( ) {

            // Tooltips are added with options.tooltips in original order.
            var tips = scope_Handles.map(addTooltip);

            bindEvent('update', function(values, handleNumber, unencoded) {

                if ( !tips[handleNumber] ) {
                    return;
                }

                var formattedValue = values[handleNumber];

                if ( options.tooltips[handleNumber] !== true ) {
                    formattedValue = options.tooltips[handleNumber].to(unencoded[handleNumber]);
                }

                tips[handleNumber].innerHTML = formattedValue;
            });
        }


        function getGroup ( mode, values, stepped ) {

            // Use the range.
            if ( mode === 'range' || mode === 'steps' ) {
                return scope_Spectrum.xVal;
            }

            if ( mode === 'count' ) {

                if ( !values ) {
                    throw new Error("noUiSlider (" + VERSION + "): 'values' required for mode 'count'.");
                }

                // Divide 0 - 100 in 'count' parts.
                var spread = ( 100 / (values - 1) );
                var v;
                var i = 0;

                values = [];

                // List these parts and have them handled as 'positions'.
                while ( (v = i++ * spread) <= 100 ) {
                    values.push(v);
                }

                mode = 'positions';
            }

            if ( mode === 'positions' ) {

                // Map all percentages to on-range values.
                return values.map(function( value ){
                    return scope_Spectrum.fromStepping( stepped ? scope_Spectrum.getStep( value ) : value );
                });
            }

            if ( mode === 'values' ) {

                // If the value must be stepped, it needs to be converted to a percentage first.
                if ( stepped ) {

                    return values.map(function( value ){

                        // Convert to percentage, apply step, return to value.
                        return scope_Spectrum.fromStepping( scope_Spectrum.getStep( scope_Spectrum.toStepping( value ) ) );
                    });

                }

                // Otherwise, we can simply use the values.
                return values;
            }
        }

        function generateSpread ( density, mode, group ) {

            function safeIncrement(value, increment) {
                // Avoid floating point variance by dropping the smallest decimal places.
                return (value + increment).toFixed(7) / 1;
            }

            var indexes = {};
            var firstInRange = scope_Spectrum.xVal[0];
            var lastInRange = scope_Spectrum.xVal[scope_Spectrum.xVal.length-1];
            var ignoreFirst = false;
            var ignoreLast = false;
            var prevPct = 0;

            // Create a copy of the group, sort it and filter away all duplicates.
            group = unique(group.slice().sort(function(a, b){ return a - b; }));

            // Make sure the range starts with the first element.
            if ( group[0] !== firstInRange ) {
                group.unshift(firstInRange);
                ignoreFirst = true;
            }

            // Likewise for the last one.
            if ( group[group.length - 1] !== lastInRange ) {
                group.push(lastInRange);
                ignoreLast = true;
            }

            group.forEach(function ( current, index ) {

                // Get the current step and the lower + upper positions.
                var step;
                var i;
                var q;
                var low = current;
                var high = group[index+1];
                var newPct;
                var pctDifference;
                var pctPos;
                var type;
                var steps;
                var realSteps;
                var stepsize;

                // When using 'steps' mode, use the provided steps.
                // Otherwise, we'll step on to the next subrange.
                if ( mode === 'steps' ) {
                    step = scope_Spectrum.xNumSteps[ index ];
                }

                // Default to a 'full' step.
                if ( !step ) {
                    step = high-low;
                }

                // Low can be 0, so test for false. If high is undefined,
                // we are at the last subrange. Index 0 is already handled.
                if ( low === false || high === undefined ) {
                    return;
                }

                // Make sure step isn't 0, which would cause an infinite loop (#654)
                step = Math.max(step, 0.0000001);

                // Find all steps in the subrange.
                for ( i = low; i <= high; i = safeIncrement(i, step) ) {

                    // Get the percentage value for the current step,
                    // calculate the size for the subrange.
                    newPct = scope_Spectrum.toStepping( i );
                    pctDifference = newPct - prevPct;

                    steps = pctDifference / density;
                    realSteps = Math.round(steps);

                    // This ratio represents the ammount of percentage-space a point indicates.
                    // For a density 1 the points/percentage = 1. For density 2, that percentage needs to be re-devided.
                    // Round the percentage offset to an even number, then divide by two
                    // to spread the offset on both sides of the range.
                    stepsize = pctDifference/realSteps;

                    // Divide all points evenly, adding the correct number to this subrange.
                    // Run up to <= so that 100% gets a point, event if ignoreLast is set.
                    for ( q = 1; q <= realSteps; q += 1 ) {

                        // The ratio between the rounded value and the actual size might be ~1% off.
                        // Correct the percentage offset by the number of points
                        // per subrange. density = 1 will result in 100 points on the
                        // full range, 2 for 50, 4 for 25, etc.
                        pctPos = prevPct + ( q * stepsize );
                        indexes[pctPos.toFixed(5)] = ['x', 0];
                    }

                    // Determine the point type.
                    type = (group.indexOf(i) > -1) ? 1 : ( mode === 'steps' ? 2 : 0 );

                    // Enforce the 'ignoreFirst' option by overwriting the type for 0.
                    if ( !index && ignoreFirst ) {
                        type = 0;
                    }

                    if ( !(i === high && ignoreLast)) {
                        // Mark the 'type' of this point. 0 = plain, 1 = real value, 2 = step value.
                        indexes[newPct.toFixed(5)] = [i, type];
                    }

                    // Update the percentage count.
                    prevPct = newPct;
                }
            });

            return indexes;
        }

        function addMarking ( spread, filterFunc, formatter ) {

            var element = document.createElement('div');
            var out = '';
            var valueSizeClasses = [
                options.cssClasses.valueNormal,
                options.cssClasses.valueLarge,
                options.cssClasses.valueSub
            ];
            var markerSizeClasses = [
                options.cssClasses.markerNormal,
                options.cssClasses.markerLarge,
                options.cssClasses.markerSub
            ];
            var valueOrientationClasses = [
                options.cssClasses.valueHorizontal,
                options.cssClasses.valueVertical
            ];
            var markerOrientationClasses = [
                options.cssClasses.markerHorizontal,
                options.cssClasses.markerVertical
            ];

            addClass(element, options.cssClasses.pips);
            addClass(element, options.ort === 0 ? options.cssClasses.pipsHorizontal : options.cssClasses.pipsVertical);

            function getClasses( type, source ){
                var a = source === options.cssClasses.value;
                var orientationClasses = a ? valueOrientationClasses : markerOrientationClasses;
                var sizeClasses = a ? valueSizeClasses : markerSizeClasses;

                return source + ' ' + orientationClasses[options.ort] + ' ' + sizeClasses[type];
            }

            function getTags( offset, source, values ) {
                return 'class="' + getClasses(values[1], source) + '" style="' + options.style + ': ' + offset + '%"';
            }

            function addSpread ( offset, values ){

                // Apply the filter function, if it is set.
                values[1] = (values[1] && filterFunc) ? filterFunc(values[0], values[1]) : values[1];

                // Add a marker for every point
                out += '<div ' + getTags(offset, options.cssClasses.marker, values) + '></div>';

                // Values are only appended for points marked '1' or '2'.
                if ( values[1] ) {
                    out += '<div ' + getTags(offset, options.cssClasses.value, values) + '>' + formatter.to(values[0]) + '</div>';
                }
            }

            // Append all points.
            Object.keys(spread).forEach(function(a){
                addSpread(a, spread[a]);
            });

            element.innerHTML = out;

            return element;
        }

        function pips ( grid ) {

            var mode = grid.mode;
            var density = grid.density || 1;
            var filter = grid.filter || false;
            var values = grid.values || false;
            var stepped = grid.stepped || false;
            var group = getGroup( mode, values, stepped );
            var spread = generateSpread( density, mode, group );
            var format = grid.format || {
                    to: Math.round
                };

            return scope_Target.appendChild(addMarking(
                spread,
                filter,
                format
            ));
        }


        // Shorthand for base dimensions.
        function baseSize ( ) {
            var rect = scope_Base.getBoundingClientRect(), alt = 'offset' + ['Width', 'Height'][options.ort];
            return options.ort === 0 ? (rect.width||scope_Base[alt]) : (rect.height||scope_Base[alt]);
        }

        // Handler for attaching events trough a proxy.
        function attachEvent ( events, element, callback, data ) {

            // This function can be used to 'filter' events to the slider.
            // element is a node, not a nodeList

            var method = function ( e ){

                if ( scope_Target.hasAttribute('disabled') ) {
                    return false;
                }

                // Stop if an active 'tap' transition is taking place.
                if ( hasClass(scope_Target, options.cssClasses.tap) ) {
                    return false;
                }

                e = fixEvent(e, data.pageOffset);

                // Handle reject of multitouch
                if ( !e ) {
                    return false;
                }

                // Ignore right or middle clicks on start #454
                if ( events === actions.start && e.buttons !== undefined && e.buttons > 1 ) {
                    return false;
                }

                // Ignore right or middle clicks on start #454
                if ( data.hover && e.buttons ) {
                    return false;
                }

                e.calcPoint = e.points[ options.ort ];

                // Call the event handler with the event [ and additional data ].
                callback ( e, data );
            };

            var methods = [];

            // Bind a closure on the target for every event type.
            events.split(' ').forEach(function( eventName ){
                element.addEventListener(eventName, method, false);
                methods.push([eventName, method]);
            });

            return methods;
        }

        // Provide a clean event with standardized offset values.
        function fixEvent ( e, pageOffset ) {

            // Prevent scrolling and panning on touch events, while
            // attempting to slide. The tap event also depends on this.
            e.preventDefault();

            // Filter the event to register the type, which can be
            // touch, mouse or pointer. Offset changes need to be
            // made on an event specific basis.
            var touch = e.type.indexOf('touch') === 0;
            var mouse = e.type.indexOf('mouse') === 0;
            var pointer = e.type.indexOf('pointer') === 0;
            var x;
            var y;

            // IE10 implemented pointer events with a prefix;
            if ( e.type.indexOf('MSPointer') === 0 ) {
                pointer = true;
            }

            if ( touch ) {

                // Fix bug when user touches with two or more fingers on mobile devices.
                // It's useful when you have two or more sliders on one page,
                // that can be touched simultaneously.
                // #649, #663, #668
                if ( e.touches.length > 1 ) {
                    return false;
                }

                // noUiSlider supports one movement at a time,
                // so we can select the first 'changedTouch'.
                x = e.changedTouches[0].pageX;
                y = e.changedTouches[0].pageY;
            }

            pageOffset = pageOffset || getPageOffset();

            if ( mouse || pointer ) {
                x = e.clientX + pageOffset.x;
                y = e.clientY + pageOffset.y;
            }

            e.pageOffset = pageOffset;
            e.points = [x, y];
            e.cursor = mouse || pointer; // Fix #435

            return e;
        }

        // Translate a coordinate in the document to a percentage on the slider
        function calcPointToPercentage ( calcPoint ) {
            var location = calcPoint - offset(scope_Base, options.ort);
            var proposal = ( location * 100 ) / baseSize();
            return options.dir ? 100 - proposal : proposal;
        }

        // Find handle closest to a certain percentage on the slider
        function getClosestHandle ( proposal ) {

            var closest = 100;
            var handleNumber = false;

            scope_Handles.forEach(function(handle, index){

                // Disabled handles are ignored
                if ( handle.hasAttribute('disabled') ) {
                    return;
                }

                var pos = Math.abs(scope_Locations[index] - proposal);

                if ( pos < closest ) {
                    handleNumber = index;
                    closest = pos;
                }
            });

            return handleNumber;
        }

        // Moves handle(s) by a percentage
        // (bool, % to move, [% where handle started, ...], [index in scope_Handles, ...])
        function moveHandles ( upward, proposal, locations, handleNumbers ) {

            var proposals = locations.slice();

            var b = [!upward, upward];
            var f = [upward, !upward];

            // Copy handleNumbers so we don't change the dataset
            handleNumbers = handleNumbers.slice();

            // Check to see which handle is 'leading'.
            // If that one can't move the second can't either.
            if ( upward ) {
                handleNumbers.reverse();
            }

            // Step 1: get the maximum percentage that any of the handles can move
            if ( handleNumbers.length > 1 ) {

                handleNumbers.forEach(function(handleNumber, o) {

                    var to = checkHandlePosition(proposals, handleNumber, proposals[handleNumber] + proposal, b[o], f[o]);

                    // Stop if one of the handles can't move.
                    if ( to === false ) {
                        proposal = 0;
                    } else {
                        proposal = to - proposals[handleNumber];
                        proposals[handleNumber] = to;
                    }
                });
            }

            // If using one handle, check backward AND forward
            else {
                b = f = [true];
            }

            var state = false;

            // Step 2: Try to set the handles with the found percentage
            handleNumbers.forEach(function(handleNumber, o) {
                state = setHandle(handleNumber, locations[handleNumber] + proposal, b[o], f[o]) || state;
            });

            // Step 3: If a handle moved, fire events
            if ( state ) {
                handleNumbers.forEach(function(handleNumber){
                    fireEvent('update', handleNumber);
                    fireEvent('slide', handleNumber);
                });
            }
        }

        // External event handling
        function fireEvent ( eventName, handleNumber, tap ) {

            Object.keys(scope_Events).forEach(function( targetEvent ) {

                var eventType = targetEvent.split('.')[0];

                if ( eventName === eventType ) {
                    scope_Events[targetEvent].forEach(function( callback ) {

                        callback.call(
                            // Use the slider public API as the scope ('this')
                            scope_Self,
                            // Return values as array, so arg_1[arg_2] is always valid.
                            scope_Values.map(options.format.to),
                            // Handle index, 0 or 1
                            handleNumber,
                            // Unformatted slider values
                            scope_Values.slice(),
                            // Event is fired by tap, true or false
                            tap || false,
                            // Left offset of the handle, in relation to the slider
                            scope_Locations.slice()
                        );
                    });
                }
            });
        }


        // Fire 'end' when a mouse or pen leaves the document.
        function documentLeave ( event, data ) {
            if ( event.type === "mouseout" && event.target.nodeName === "HTML" && event.relatedTarget === null ){
                eventEnd (event, data);
            }
        }

        // Handle movement on document for handle and range drag.
        function eventMove ( event, data ) {

            // Fix #498
            // Check value of .buttons in 'start' to work around a bug in IE10 mobile (data.buttonsProperty).
            // https://connect.microsoft.com/IE/feedback/details/927005/mobile-ie10-windows-phone-buttons-property-of-pointermove-event-always-zero
            // IE9 has .buttons and .which zero on mousemove.
            // Firefox breaks the spec MDN defines.
            if ( navigator.appVersion.indexOf("MSIE 9") === -1 && event.buttons === 0 && data.buttonsProperty !== 0 ) {
                return eventEnd(event, data);
            }

            // Check if we are moving up or down
            var movement = (options.dir ? -1 : 1) * (event.calcPoint - data.startCalcPoint);

            // Convert the movement into a percentage of the slider width/height
            var proposal = (movement * 100) / data.baseSize;

            moveHandles(movement > 0, proposal, data.locations, data.handleNumbers);
        }

        // Unbind move events on document, call callbacks.
        function eventEnd ( event, data ) {

            // The handle is no longer active, so remove the class.
            if ( scope_ActiveHandle ) {
                removeClass(scope_ActiveHandle, options.cssClasses.active);
                scope_ActiveHandle = false;
            }

            // Remove cursor styles and text-selection events bound to the body.
            if ( event.cursor ) {
                document.body.style.cursor = '';
                document.body.removeEventListener('selectstart', document.body.noUiListener);
            }

            // Unbind the move and end events, which are added on 'start'.
            document.documentElement.noUiListeners.forEach(function( c ) {
                document.documentElement.removeEventListener(c[0], c[1]);
            });

            // Remove dragging class.
            removeClass(scope_Target, options.cssClasses.drag);

            setZindex();

            data.handleNumbers.forEach(function(handleNumber){
                fireEvent('set', handleNumber);
                fireEvent('change', handleNumber);
                fireEvent('end', handleNumber);
            });
        }

        // Bind move events on document.
        function eventStart ( event, data ) {

            if ( data.handleNumbers.length === 1 ) {

                var handle = scope_Handles[data.handleNumbers[0]];

                // Ignore 'disabled' handles
                if ( handle.hasAttribute('disabled') ) {
                    return false;
                }

                // Mark the handle as 'active' so it can be styled.
                scope_ActiveHandle = handle.children[0];
                addClass(scope_ActiveHandle, options.cssClasses.active);
            }

            // Fix #551, where a handle gets selected instead of dragged.
            event.preventDefault();

            // A drag should never propagate up to the 'tap' event.
            event.stopPropagation();

            // Attach the move and end events.
            var moveEvent = attachEvent(actions.move, document.documentElement, eventMove, {
                startCalcPoint: event.calcPoint,
                baseSize: baseSize(),
                pageOffset: event.pageOffset,
                handleNumbers: data.handleNumbers,
                buttonsProperty: event.buttons,
                locations: scope_Locations.slice()
            });

            var endEvent = attachEvent(actions.end, document.documentElement, eventEnd, {
                handleNumbers: data.handleNumbers
            });

            var outEvent = attachEvent("mouseout", document.documentElement, documentLeave, {
                handleNumbers: data.handleNumbers
            });

            document.documentElement.noUiListeners = moveEvent.concat(endEvent, outEvent);

            // Text selection isn't an issue on touch devices,
            // so adding cursor styles can be skipped.
            if ( event.cursor ) {

                // Prevent the 'I' cursor and extend the range-drag cursor.
                document.body.style.cursor = getComputedStyle(event.target).cursor;

                // Mark the target with a dragging state.
                if ( scope_Handles.length > 1 ) {
                    addClass(scope_Target, options.cssClasses.drag);
                }

                var f = function(){
                    return false;
                };

                document.body.noUiListener = f;

                // Prevent text selection when dragging the handles.
                document.body.addEventListener('selectstart', f, false);
            }

            data.handleNumbers.forEach(function(handleNumber){
                fireEvent('start', handleNumber);
            });
        }

        // Move closest handle to tapped location.
        function eventTap ( event ) {

            // The tap event shouldn't propagate up
            event.stopPropagation();

            var proposal = calcPointToPercentage(event.calcPoint);
            var handleNumber = getClosestHandle(proposal);

            // Tackle the case that all handles are 'disabled'.
            if ( handleNumber === false ) {
                return false;
            }

            // Flag the slider as it is now in a transitional state.
            // Transition takes a configurable amount of ms (default 300). Re-enable the slider after that.
            if ( !options.events.snap ) {
                addClassFor(scope_Target, options.cssClasses.tap, options.animationDuration);
            }

            setHandle(handleNumber, proposal, true, true);

            setZindex();

            fireEvent('slide', handleNumber, true);
            fireEvent('set', handleNumber, true);
            fireEvent('change', handleNumber, true);
            fireEvent('update', handleNumber, true);

            if ( options.events.snap ) {
                eventStart(event, { handleNumbers: [handleNumber] });
            }
        }

        // Fires a 'hover' event for a hovered mouse/pen position.
        function eventHover ( event ) {

            var proposal = calcPointToPercentage(event.calcPoint);

            var to = scope_Spectrum.getStep(proposal);
            var value = scope_Spectrum.fromStepping(to);

            Object.keys(scope_Events).forEach(function( targetEvent ) {
                if ( 'hover' === targetEvent.split('.')[0] ) {
                    scope_Events[targetEvent].forEach(function( callback ) {
                        callback.call( scope_Self, value );
                    });
                }
            });
        }

        // Attach events to several slider parts.
        function bindSliderEvents ( behaviour ) {

            // Attach the standard drag event to the handles.
            if ( !behaviour.fixed ) {

                scope_Handles.forEach(function( handle, index ){

                    // These events are only bound to the visual handle
                    // element, not the 'real' origin element.
                    attachEvent ( actions.start, handle.children[0], eventStart, {
                        handleNumbers: [index]
                    });
                });
            }

            // Attach the tap event to the slider base.
            if ( behaviour.tap ) {
                attachEvent (actions.start, scope_Base, eventTap, {});
            }

            // Fire hover events
            if ( behaviour.hover ) {
                attachEvent (actions.move, scope_Base, eventHover, { hover: true });
            }

            // Make the range draggable.
            if ( behaviour.drag ){

                scope_Connects.forEach(function( connect, index ){

                    if ( connect === false || index === 0 || index === scope_Connects.length - 1 ) {
                        return;
                    }

                    var handleBefore = scope_Handles[index - 1];
                    var handleAfter = scope_Handles[index];
                    var eventHolders = [connect];

                    addClass(connect, options.cssClasses.draggable);

                    // When the range is fixed, the entire range can
                    // be dragged by the handles. The handle in the first
                    // origin will propagate the start event upward,
                    // but it needs to be bound manually on the other.
                    if ( behaviour.fixed ) {
                        eventHolders.push(handleBefore.children[0]);
                        eventHolders.push(handleAfter.children[0]);
                    }

                    eventHolders.forEach(function( eventHolder ) {
                        attachEvent ( actions.start, eventHolder, eventStart, {
                            handles: [handleBefore, handleAfter],
                            handleNumbers: [index - 1, index]
                        });
                    });
                });
            }
        }


        // Split out the handle positioning logic so the Move event can use it, too
        function checkHandlePosition ( reference, handleNumber, to, lookBackward, lookForward ) {

            // For sliders with multiple handles, limit movement to the other handle.
            // Apply the margin option by adding it to the handle positions.
            if ( scope_Handles.length > 1 ) {

                if ( lookBackward && handleNumber > 0 ) {
                    to = Math.max(to, reference[handleNumber - 1] + options.margin);
                }

                if ( lookForward && handleNumber < scope_Handles.length - 1 ) {
                    to = Math.min(to, reference[handleNumber + 1] - options.margin);
                }
            }

            // The limit option has the opposite effect, limiting handles to a
            // maximum distance from another. Limit must be > 0, as otherwise
            // handles would be unmoveable.
            if ( scope_Handles.length > 1 && options.limit ) {

                if ( lookBackward && handleNumber > 0 ) {
                    to = Math.min(to, reference[handleNumber - 1] + options.limit);
                }

                if ( lookForward && handleNumber < scope_Handles.length - 1 ) {
                    to = Math.max(to, reference[handleNumber + 1] - options.limit);
                }
            }

            // The padding option keeps the handles a certain distance from the
            // edges of the slider. Padding must be > 0.
            if ( options.padding ) {

                if ( handleNumber === 0 ) {
                    to = Math.max(to, options.padding);
                }

                if ( handleNumber === scope_Handles.length - 1 ) {
                    to = Math.min(to, 100 - options.padding);
                }
            }

            to = scope_Spectrum.getStep(to);

            // Limit percentage to the 0 - 100 range
            to = limit(to);

            // Return false if handle can't move
            if ( to === reference[handleNumber] ) {
                return false;
            }

            return to;
        }

        function toPct ( pct ) {
            return pct + '%';
        }

        // Updates scope_Locations and scope_Values, updates visual state
        function updateHandlePosition ( handleNumber, to ) {

            // Update locations.
            scope_Locations[handleNumber] = to;

            // Convert the value to the slider stepping/range.
            scope_Values[handleNumber] = scope_Spectrum.fromStepping(to);

            // Called synchronously or on the next animationFrame
            var stateUpdate = function() {
                scope_Handles[handleNumber].style[options.style] = toPct(to);
                updateConnect(handleNumber);
                updateConnect(handleNumber + 1);
            };

            // Set the handle to the new position.
            // Use requestAnimationFrame for efficient painting.
            // No significant effect in Chrome, Edge sees dramatic performace improvements.
            // Option to disable is useful for unit tests, and single-step debugging.
            if ( window.requestAnimationFrame && options.useRequestAnimationFrame ) {
                window.requestAnimationFrame(stateUpdate);
            } else {
                stateUpdate();
            }
        }

        function setZindex ( ) {

            scope_HandleNumbers.forEach(function(handleNumber){
                // Handles before the slider middle are stacked later = higher,
                // Handles after the middle later is lower
                // [[7] [8] .......... | .......... [5] [4]
                var dir = (scope_Locations[handleNumber] > 50 ? -1 : 1);
                var zIndex = 3 + (scope_Handles.length + (dir * handleNumber));
                scope_Handles[handleNumber].childNodes[0].style.zIndex = zIndex;
            });
        }

        // Test suggested values and apply margin, step.
        function setHandle ( handleNumber, to, lookBackward, lookForward ) {

            to = checkHandlePosition(scope_Locations, handleNumber, to, lookBackward, lookForward);

            if ( to === false ) {
                return false;
            }

            updateHandlePosition(handleNumber, to);

            return true;
        }

        // Updates style attribute for connect nodes
        function updateConnect ( index ) {

            // Skip connects set to false
            if ( !scope_Connects[index] ) {
                return;
            }

            var l = 0;
            var h = 100;

            if ( index !== 0 ) {
                l = scope_Locations[index - 1];
            }

            if ( index !== scope_Connects.length - 1 ) {
                h = scope_Locations[index];
            }

            scope_Connects[index].style[options.style] = toPct(l);
            scope_Connects[index].style[options.styleOposite] = toPct(100 - h);
        }

        // ...
        function setValue ( to, handleNumber ) {

            // Setting with null indicates an 'ignore'.
            // Inputting 'false' is invalid.
            if ( to === null || to === false ) {
                return;
            }

            // If a formatted number was passed, attemt to decode it.
            if ( typeof to === 'number' ) {
                to = String(to);
            }

            to = options.format.from(to);

            // Request an update for all links if the value was invalid.
            // Do so too if setting the handle fails.
            if ( to !== false && !isNaN(to) ) {
                setHandle(handleNumber, scope_Spectrum.toStepping(to), false, false);
            }
        }

        // Set the slider value.
        function valueSet ( input, fireSetEvent ) {

            var values = asArray(input);
            var isInit = scope_Locations[0] === undefined;

            // Event fires by default
            fireSetEvent = (fireSetEvent === undefined ? true : !!fireSetEvent);

            values.forEach(setValue);

            // Animation is optional.
            // Make sure the initial values were set before using animated placement.
            if ( options.animate && !isInit ) {
                addClassFor(scope_Target, options.cssClasses.tap, options.animationDuration);
            }

            // Now that all base values are set, apply constraints
            scope_HandleNumbers.forEach(function(handleNumber){
                setHandle(handleNumber, scope_Locations[handleNumber], true, false);
            });

            setZindex();

            scope_HandleNumbers.forEach(function(handleNumber){

                fireEvent('update', handleNumber);

                // Fire the event only for handles that received a new value, as per #579
                if ( values[handleNumber] !== null && fireSetEvent ) {
                    fireEvent('set', handleNumber);
                }
            });
        }

        // Reset slider to initial values
        function valueReset ( fireSetEvent ) {
            valueSet(options.start, fireSetEvent);
        }

        // Get the slider value.
        function valueGet ( ) {

            var values = scope_Values.map(options.format.to);

            // If only one handle is used, return a single value.
            if ( values.length === 1 ){
                return values[0];
            }

            return values;
        }

        // Removes classes from the root and empties it.
        function destroy ( ) {

            for ( var key in options.cssClasses ) {
                if ( !options.cssClasses.hasOwnProperty(key) ) { continue; }
                removeClass(scope_Target, options.cssClasses[key]);
            }

            while (scope_Target.firstChild) {
                scope_Target.removeChild(scope_Target.firstChild);
            }

            delete scope_Target.noUiSlider;
        }

        // Get the current step size for the slider.
        function getCurrentStep ( ) {

            // Check all locations, map them to their stepping point.
            // Get the step point, then find it in the input list.
            return scope_Locations.map(function( location, index ){

                var nearbySteps = scope_Spectrum.getNearbySteps( location );
                var value = scope_Values[index];
                var increment = nearbySteps.thisStep.step;
                var decrement = null;

                // If the next value in this step moves into the next step,
                // the increment is the start of the next step - the current value
                if ( increment !== false ) {
                    if ( value + increment > nearbySteps.stepAfter.startValue ) {
                        increment = nearbySteps.stepAfter.startValue - value;
                    }
                }


                // If the value is beyond the starting point
                if ( value > nearbySteps.thisStep.startValue ) {
                    decrement = nearbySteps.thisStep.step;
                }

                else if ( nearbySteps.stepBefore.step === false ) {
                    decrement = false;
                }

                // If a handle is at the start of a step, it always steps back into the previous step first
                else {
                    decrement = value - nearbySteps.stepBefore.highestStep;
                }


                // Now, if at the slider edges, there is not in/decrement
                if ( location === 100 ) {
                    increment = null;
                }

                else if ( location === 0 ) {
                    decrement = null;
                }

                // As per #391, the comparison for the decrement step can have some rounding issues.
                var stepDecimals = scope_Spectrum.countStepDecimals();

                // Round per #391
                if ( increment !== null && increment !== false ) {
                    increment = Number(increment.toFixed(stepDecimals));
                }

                if ( decrement !== null && decrement !== false ) {
                    decrement = Number(decrement.toFixed(stepDecimals));
                }

                return [decrement, increment];
            });
        }

        // Attach an event to this slider, possibly including a namespace
        function bindEvent ( namespacedEvent, callback ) {
            scope_Events[namespacedEvent] = scope_Events[namespacedEvent] || [];
            scope_Events[namespacedEvent].push(callback);

            // If the event bound is 'update,' fire it immediately for all handles.
            if ( namespacedEvent.split('.')[0] === 'update' ) {
                scope_Handles.forEach(function(a, index){
                    fireEvent('update', index);
                });
            }
        }

        // Undo attachment of event
        function removeEvent ( namespacedEvent ) {

            var event = namespacedEvent && namespacedEvent.split('.')[0];
            var namespace = event && namespacedEvent.substring(event.length);

            Object.keys(scope_Events).forEach(function( bind ){

                var tEvent = bind.split('.')[0],
                    tNamespace = bind.substring(tEvent.length);

                if ( (!event || event === tEvent) && (!namespace || namespace === tNamespace) ) {
                    delete scope_Events[bind];
                }
            });
        }

        // Updateable: margin, limit, padding, step, range, animate, snap
        function updateOptions ( optionsToUpdate, fireSetEvent ) {

            // Spectrum is created using the range, snap, direction and step options.
            // 'snap' and 'step' can be updated, 'direction' cannot, due to event binding.
            // If 'snap' and 'step' are not passed, they should remain unchanged.
            var v = valueGet();

            var updateAble = ['margin', 'limit', 'padding', 'range', 'animate', 'snap', 'step', 'format'];

            // Only change options that we're actually passed to update.
            updateAble.forEach(function(name){
                if ( optionsToUpdate[name] !== undefined ) {
                    originalOptions[name] = optionsToUpdate[name];
                }
            });

            var newOptions = testOptions(originalOptions);

            // Load new options into the slider state
            updateAble.forEach(function(name){
                if ( optionsToUpdate[name] !== undefined ) {
                    options[name] = newOptions[name];
                }
            });

            // Save current spectrum direction as testOptions in testRange call
            // doesn't rely on current direction
            newOptions.spectrum.direction = scope_Spectrum.direction;
            scope_Spectrum = newOptions.spectrum;

            // Limit, margin and padding depend on the spectrum but are stored outside of it. (#677)
            options.margin = newOptions.margin;
            options.limit = newOptions.limit;
            options.padding = newOptions.padding;

            // Invalidate the current positioning so valueSet forces an update.
            scope_Locations = [];
            valueSet(optionsToUpdate.start || v, fireSetEvent);
        }

        // Throw an error if the slider was already initialized.
        if ( scope_Target.noUiSlider ) {
            throw new Error("noUiSlider (" + VERSION + "): Slider was already initialized.");
        }

        // Create the base element, initialise HTML and set classes.
        // Add handles and connect elements.
        addSlider(scope_Target);
        addElements(options.connect, scope_Base);

        scope_Self = {
            destroy: destroy,
            steps: getCurrentStep,
            on: bindEvent,
            off: removeEvent,
            get: valueGet,
            set: valueSet,
            reset: valueReset,
            // Exposed for unit testing, don't use this in your application.
            __moveHandles: function(a, b, c) { moveHandles(a, b, scope_Locations, c); },
            options: originalOptions, // Issue #600, #678
            updateOptions: updateOptions,
            target: scope_Target, // Issue #597
            pips: pips // Issue #594
        };

        // Attach user events.
        bindSliderEvents(options.events);

        // Use the public value method to set the start values.
        valueSet(options.start);

        if ( options.pips ) {
            pips(options.pips);
        }

        if ( options.tooltips ) {
            tooltips();
        }

        return scope_Self;

    }


    // Run the standard initializer
    function initialize ( target, originalOptions ) {

        if ( !target.nodeName ) {
            throw new Error("noUiSlider (" + VERSION + "): create requires a single element.");
        }

        // Test the options and create the slider environment;
        var options = testOptions( originalOptions, target );
        var api = closure( target, options, originalOptions );

        target.noUiSlider = api;

        return api;
    }

    // Use an object instead of a function for future expansibility;
    return {
        version: VERSION,
        create: initialize
    };

}));
// fine nouislider

'use strict';
var start = 0;
var rows = 20;

var sort = 'score';
var direction = 'desc';

var total_products = 0;
var results_loaded = 0;
var canScroll = true;

var blurTimeout = null;

var userLang = navigator.language || navigator.userLanguage;

var localeLabels = {
	en: {
		resultsCount: '<b>{0}</b> Results Found',
		activeFilters: 'Active Filters',
		removeFilters: 'Remove Filters',
		showMore: 'Show more',
		showLess: 'Show less',
		noResults: 'Sorry, no results found.',
		didYouMean: 'Did you mean:',
		from: 'From',
		to: 'To'
	},
	it: {
		resultsCount: '<b>{0}</b> Risultati Trovati',
		activeFilters: 'Filtri Attivi',
		removeFilters: 'Elimina Filtri',
		showMore: 'Mostra tutto',
		showLess: 'Mostra meno',
		noResults: 'Siamo spiacenti, non Ã© stato trovato nessun risultato.',
		didYouMean: 'Forse cercavi:',
		from: 'Da',
		to: 'a'
	},
	fr: {
		resultsCount: '<b>{0}</b> RÃ©sultats trouvÃ©s',
		activeFilters: 'Filtres actifs',
		removeFilters: 'Ã‰liminer Filtres',
		showMore: 'Afficher tout',
		showLess: 'Afficher moins',
		noResults: 'Nous sommes dÃ©solÃ©s, mais nous n\'avons trouvÃ© aucun rÃ©sultat.',
		didYouMean: 'Peut-Ãªtre cherchiez-vous:',
		from: 'A',
		to: 'Ã '
	},
	es: {
		resultsCount: '<b>{0}</b> Resultados de la bÃºsqueda',
		activeFilters: 'Filtros Activos',
		removeFilters: 'Elimina Filtros',
		showMore: 'Muestra MÃ¡s',
		showLess: 'Muestra Menos',
		noResults: 'Â¡Disculpe! no  encontraron  resultados',
		didYouMean: 'QuizÃ¡s estabas buscando:',
		from: 'Desde',
		to: 'a'
	}
}

var cVettaLabels = cVettaLabels || localeLabels[userLang.substring(0,2)]; // se presenti le etichette sul sito prendo quelle, altrimenti quelle di default
var facet_applied = {};

function loadAdvertise(){
	// svuoto eventuali banner pubblicitari
	document.getElementById('cvettaAdvertiseList').innerHTML = '';

	var parameters = '';
	var str = document.getElementById("cvetta-search").value;
	var url = '//ws10b.cvetta.io/advertise.php?qt=advertise&lang='+cVettaParams.lang+'&q='+encodeURIComponent(str);

	// per forzare l'invio di una vetrina indipendentemente dal dominio (es da ittweb.net)
	if (cVettaParams.idvetrina) {
		url += '&idvetrina='+cVettaParams.idvetrina;
	}

	var script = document.createElement('script');
	script.src = url;
	script.onload = function () {
		this.remove();
	};
	document.body.appendChild(script);
}

function loadData(st){
	canScroll = false;

	start = st;

    // rimuovo i campi hidden della form
    var form = document.getElementById("cVetta-form");
    var hidden_inputs = form.getElementsByClassName('cVetta-hidden-input');
    //console.log("hidden_inputs:");
    //console.log(hidden_inputs);
    var hidden_inputs_lenght = hidden_inputs.length;
    //console.log('hidden_inputs_lenght:'+hidden_inputs_lenght);
    if (hidden_inputs_lenght > 0) {
        for (i = 0; i < hidden_inputs_lenght; i++) {
            //console.log("rimuovo: "+hidden_inputs[0]);
            //if (hidden_inputs[i] != undefined) {
                form.removeChild(hidden_inputs[0]);
            //}
        }
    }

	var str = document.getElementById("cvetta-search").value;
	var parameters = '';
	for (var i in facet_applied) {
		//console.log(i);
		//console.log(facet_applied[i]);
		var values = facet_applied[i];
		if( typeof values === 'string' ) {
			values = [ values ];
		}
		for (var value = 0; value < values.length; value++) {
			parameters += '|ff=' + i + ':' + values[value];
            //console.log("addHidden facets_label[]:"+i);
            //console.log("addHidden facets_value[]:"+values[value]);
            addHidden(form, 'facets_label[]', i);
            addHidden(form, 'facets_value[]', values[value]);
		}
	}

    // salvo la ricerca nei cookie
    document.cookie = "cvetta.search.text="+str;
    document.cookie = "cvetta.search.facets="+JSON.stringify(facet_applied);

	var url = '//ws10b.cvetta.io/search.php?qt=find&lang='+cVettaParams.lang+'&q='+encodeURIComponent(str)+'&sort='+sort+'+'+direction+'&params='+encodeURIComponent(parameters)+'&start='+start+'&rows='+rows;

	// per forzare l'invio di una vetrina indipendentemente dal dominio (es da ittweb.net)
	if (cVettaParams.idvetrina) {
		url += '&idvetrina='+cVettaParams.idvetrina;
	}

	var script = document.createElement('script');
	script.src = url;
	script.onload = function () {
		this.remove();
	};
	document.body.appendChild(script);

	var body = document.getElementsByTagName('body')[0];
	body.className = body.className.replace(/(?:^|\s)cvetta-search-active(?!\S)/g, '');
	body.className += ' cvetta-search-active';
}

function getResults(data) {
	if (start > 0) {
		var docs_lenght = data.response.docs.length;
		var bufferContent = '';
		for (var i = 0; i < docs_lenght; i++) {
			var item = data.response.docs[i];
			bufferContent += templateProduct(data, item);
		};
		document.getElementById('cvettaResultList').innerHTML = document.getElementById('cvettaResultList').innerHTML + bufferContent;
		results_loaded += i;
		canScroll = true;

		runListeners();

        var event = new CustomEvent('cVettaGetResults', { 'detail': data });
        document.dispatchEvent(event);
        console.log('dispateched cVettaGetResults');

		return;
	}

	results_loaded = 0; // nuova ricerca: azzero il numero di risultati caricati
	document.getElementById("cvettaResultList").scrollTop = 0; //nuova ricerca: porto lo scroll in alto altrimenti parte in automatico l'infinite scroll

	//console.log(data);

	document.getElementById('cvetta').style.display = 'block';
	var bufferContent = '';
	document.getElementById('cvettaDidYouMeanContainer').innerHTML = '';
	document.getElementById('cvettaSuggestContainer').innerHTML = '';

	start = data.responseHeader.params.start;
	rows = data.responseHeader.params.rows;
	total_products = data.response.numFound;

	facet_applied = {};
	var facet_applied_count = 0;

	// riempio l'oggetto con i filtri applicati
	if (data.responseHeader.params.fq) {
		//console.log(JSON.parse(data.responseHeader.params.fq));
		//controllo se Ã¨ un array
		var fq = data.responseHeader.params.fq;
		if( typeof fq === 'string' ) {
			fq = [ fq ];
		}
		var fq_lenght = fq.length;
		for (i = 0; i < fq_lenght; i++) {
			var fq_facet = fq[i].split(':');
			var fq_facet_name = fq_facet[0];
			var fq_facet_value = fq_facet[1];

			// tolgo le parentesi e le virgolette al valore della faccetta
			var fq_facet_value = fq_facet_value.split(' AND ');
			facet_applied[fq_facet_name] = [];
			var fq_facet_value_lenght = fq_facet_value.length;
			for (var k = 0; k < fq_facet_value_lenght; k++) {
				var fq_facet_value2 = fq_facet_value[k].split(' OR ');
				var fq_facet_value2_lenght = fq_facet_value2.length;
				for (var z = 0; z < fq_facet_value2_lenght; z++) {
					var fq_facet_value3 = fq_facet_value2[z];

					if (fq_facet_value3[fq_facet_value3.length - 1] == ')') {
						fq_facet_value3 = fq_facet_value3.substring(0, fq_facet_value3.length - 1);
					}
					if (fq_facet_value3[fq_facet_value3.length - 1] == '"') {
						fq_facet_value3 = fq_facet_value3.substring(0, fq_facet_value3.length - 1);
					}
					if (fq_facet_value3[0] == '(') {
						fq_facet_value3 = fq_facet_value3.substring(1, fq_facet_value3.length);
					}
					if (fq_facet_value3[0] == '"') {
						fq_facet_value3 = fq_facet_value3.substring(1, fq_facet_value3.length);
					}

					facet_applied[fq_facet_name].push(fq_facet_value3);
				}

				facet_applied_count++;
			}
			//
		}
		//console.log(facet_applied);
	}
	//

	// accendo il menÃ¹ dei filtri nella versione mobile
	var cvettaOffcanvasNavTriggerLabel = document.getElementById("cvetta-offcanvas-nav-trigger-label");
	cvettaOffcanvasNavTriggerLabel.className = cvettaOffcanvasNavTriggerLabel.className.replace(/(?:^|\s)active(?!\S)/g, '');
	if (facet_applied_count > 0) {
		cvettaOffcanvasNavTriggerLabel.className += ' active';
	}

	document.getElementById('cvettaResultsCount').innerHTML = cVettaLabels['resultsCount'].format(data.response.numFound);
	document.getElementById('cvettaResultsCountMobile').innerHTML = cVettaLabels['resultsCount'].format(data.response.numFound);

	var cvetta_lang = cVettaParams.lang.substring(0,2);

	if( data.response.numFound > 0 ){

		/* content */
		//console.log(data.response.docs);
		var docs_lenght = data.response.docs.length;
		for (i = 0; i < docs_lenght; i++) {
			var item = data.response.docs[i];
			bufferContent += templateProduct(data, item);
		};
		results_loaded += i;
		/* faccette */
		var bufferFacets = '<ul class="facets">';
		//var facetFilterAppliedCount = countProperties(data.facetfilterapplied);
		if(facet_applied_count > 0){
			bufferFacets += '<li><a href="#">'+cVettaLabels['activeFilters']+'</a><span id="cvetta-count-filters">'+facet_applied_count+'</span><a id="cvetta-clear-filters" href="#" title="' + cVettaLabels["removeFilters"] + '"></a><ul class="sub selected">';

			for (var facet in facet_applied) {

				var item = facet_applied[facet];

				for (var index_facet = 0; index_facet < item.length; index_facet++) {
					var value_facet = item[index_facet];
					var value_facet_display = value_facet;

					if (value_facet[0] == '[' && value_facet[value_facet.length -1] == ']' && value_facet.indexOf(' TO ')) {
						var range = value_facet.split(' TO ');
						if (range[0][0] == '[') {
							range[0] = range[0].substring(1, range[0].length);
						}
						if (range[1][range[1].length - 1] == ']') {
							range[1] = range[1].substring(0, range[1].length - 1);
						}

						if(cvetta_lang == 'it' || cVettaParams.price_separator == ',')  {
							var low_range = numberWithCommas(range[0]);
							var upper_range = numberWithCommas(range[1]);
						} else {
							var low_range = range[0];
							var upper_range = range[1];
						}

						value_facet_display = low_range + ' - ' + upper_range;
					}

					var facetLabel = cVettaFacetLabels[facet] ? cVettaFacetLabels[facet] : facet;
					bufferFacets += '<li><a href="#" class="facet_applied" data-facet-name="' + facet + '" data-facet-value="' + value_facet + '" ><span class="facet-close-left"><strong>' + facetLabel + '</strong> > ' + value_facet_display + '</span><span class="facet-close-right"></span></a></li>';

				};

			};
			bufferFacets += '</ul></li>';
		}
		if(typeof data.facet_counts.facet_fields != 'undefined'){
            //console.log(data.facet_counts.facet_fields)
			for (var facet in data.facet_counts.facet_fields) {

               

                var item_tmp = data.facet_counts.facet_fields[facet];
               // console.log(item_tmp);

               var keySorted = Object.keys(item_tmp).sort(function(a, b) { return item_tmp[b] - item_tmp[a] }) 

               
               /*var item = {}
               keySorted.forEach(function(key) {
                console.log(key, item_tmp[key]);
                    item[] = { key: item_tmp[key] }
                });
                    */
                

                //console.log(item)
				var facetShow = false; // prima di mostrare la faccetta devo verificare che abbia almeno una opzione selezionabile
				var facetLabel = cVettaFacetLabels[facet] ? cVettaFacetLabels[facet] : facet;
				var facetTitle = '<li><a role="button" class="cvetta-facet cvetta-minus" href="#">' + facetLabel + '</a><ul class="cvetta-collapse sub" >';
				var facetOptions = '';

				var facetOptionsCount = countProperties(item);
				if (facetOptionsCount < 1) continue;

				//for (var key in item) {
                keySorted.forEach(function(key) {
					//var val = item[key];
					var val = item_tmp[key];
					var relInfo = '|ff=' + facet + ':' + key;
					/*console.log(data.facetfilterapplied);
					console.log(data.facetfilterapplied.facet);
					console.log(facet);
					console.log(key);*/
					var faccetta_selezionata = false;
					//console.log(data.facetfilterapplied[facet][key]);
					if(typeof facet_applied[facet] != 'undefined') {
						var length = facet_applied[facet].length;
						for(var i = 0; i < length; i++) {
							if(facet_applied[facet][i] == key) {
								faccetta_selezionata = true;
							}
						}
					}
					if(!faccetta_selezionata) {
						facetShow = true;
						facetOptions += '<li><a href="#" class="facet_field" data-facet-name="' + facet + '" data-facet-value="' + key + '">' + key + ' (' + val + ')</a></li>';
					}
				});

				if (facetShow) {
					bufferFacets += facetTitle;
					bufferFacets += facetOptions;
					bufferFacets += '<li><a href="#" class="view-more-subact" title="'+cVettaLabels["showMore"]+'"></a></li>'; // view more button
					bufferFacets += '</ul></li>';
				}
			};
		}
		var facet_ranges_count = countProperties(data.facet_counts.facet_ranges);
		//if(typeof data.facet_counts.facet_ranges != 'undefined'){
		var slider_ranges = [];
		if (facet_ranges_count > 0) {
			for (var facet in data.facet_counts.facet_ranges) {
				var item = data.facet_counts.facet_ranges[facet];
				var facetLabel = cVettaFacetLabels[facet] ? cVettaFacetLabels[facet] : facet;
				bufferFacets += '<li><a role="button" class="cvetta-facet cvetta-minus" href="#">' + facetLabel + '</a><ul class="cvetta-collapse sub" >';
				//var facetRange = true;
				var facetRangeLower = Math.floor(parseFloat(item.start) / parseFloat(item.gap)) * parseFloat(item.gap);
				var facetRangeUpper = Math.ceil(parseFloat(item.end) / parseFloat(item.gap)) * parseFloat(item.gap);

				var facetRangeLowerStart = facetRangeLower;
				var facetRangeUpperEnd = facetRangeUpper;

				if (facet_applied[facet]) {
					var range = facet_applied[facet];
					range = range[0].substring(1, range[0].length -1);
					range = range.split(' TO ');
					facetRangeLowerStart = range[0]; //Math.floor(range[0]);
					facetRangeUpperEnd = range[1]; //Math.ceil(range[1]);
				}

				if(cvetta_lang == 'it' || cVettaParams.price_separator == ',')  {
					var lower_range_value = numberWithCommas(facetRangeLowerStart);
					var upper_range_value = numberWithCommas(facetRangeUpperEnd);
				} else {
					var lower_range_value = facetRangeLowerStart;
					var upper_range_value = facetRangeUpperEnd;
				}

				bufferFacets += '<li><div id="'+facet+'_slider" class="facet_slider"></div><div class="facet_slider_info">'+cVettaLabels["from"]+' <span id="lower_'+facet+'_info">'+ lower_range_value + '</span> '+cVettaLabels["to"]+' <span id="upper_'+facet+'_info">'+ upper_range_value + '</span></div><input id="lower_'+facet+'" type="hidden" value="'+ facetRangeLower + '" readonly="readonly" /><input id="upper_'+facet+'" type="hidden" value="' + facetRangeUpperEnd +'" readonly="readonly"/></li>';

				slider_ranges.push({facet: facet, facetRangeLower: facetRangeLower, facetRangeUpper: facetRangeUpper, facetRangeLowerStart: facetRangeLowerStart, facetRangeUpperEnd: facetRangeUpperEnd, step: item.gap});

				/*} else {
					for (var key in item.counts) {
						var val = item.counts[key];
						var minValue = parseFloat(key) - parseFloat(item.gap);
						//if (minValue < 0) minValue = 0;
						bufferFacets += '<li><a href="#" class="facet_range" data-facet-name="' + facet + '" data-facet-value="['+minValue+'+TO+'+key+']" >' + key + ' (' + val + ')</a></li>';
					};
				}*/
				bufferFacets += '</ul></li>';

			};
		}
		bufferFacets += '</ul>';
		/* pivot */
		/*if(typeof data.facet_counts.facet_pivot != 'undefined'){
		//console.log(data.facet_counts);
		jQuery.each( data.facet_counts.facet_pivot, function( facet, item ) {
		//console.log( item );
		jQuery.each( item, function( keyCat, category ) {
		counter++;
		bufferFacets += '<li>' + category.value + ' (' + category.count + ' record)</li><ul class="sub subtac' + counter + '">';
		jQuery.each( category.pivot, function( keySubCat, subcategory ) {
		var relInfo = parameters + '|fq=' + subcategory.value;
		bufferFacets += '<li><a href="' + str + '" class="facet" rel="' + relInfo + '">' + subcategory.value + ' (' + subcategory.count + ' record)</a></li>';
		});
		bufferFacets += '</ul>';
		});
		});
		}

		bufferFacets += '</ul>';*/

		document.getElementById('cvettaResultList').innerHTML = bufferContent;
		document.getElementById('cvettaFacetContainer').innerHTML = bufferFacets;

		for (var i = 0; i < slider_ranges.length; i++) {
			setupFacetSlider(slider_ranges[i].facet, slider_ranges[i].facetRangeLower, slider_ranges[i].facetRangeUpper, slider_ranges[i].facetRangeLowerStart, slider_ranges[i].facetRangeUpperEnd, slider_ranges[i].step);
		}

		canScroll = true;

	}
	else { // se non ho risutati

		// svuoto i contenuti
		var bufferContent = '<p class="noresults">'+cVettaLabels["noResults"]+'</p>';
		var bufferFacets = '<ul class="facets">';
		//var facet_applied_count = countProperties(facet_applied);
		if(facet_applied_count > 0){
			bufferFacets += '<li><a href="#">'+cVettaLabels['activeFilters']+'</a><span id="cvetta-count-filters">'+facet_applied_count+'</span><a id="cvetta-clear-filters" href="#" title="' + cVettaLabels["removeFilters"] + '"></a><ul class="sub selected">';

			for (var facet in facet_applied) {

				var item = facet_applied[facet];

				//jQuery.each( item, function( index_facet, value_facet ) {
				for (var index_facet = 0; index_facet < item.length; index_facet++) {
					var value_facet = item[index_facet];
					var value_facet_display = value_facet;

					if (value_facet[0] == '[' && value_facet[value_facet.length -1] == ']' && value_facet.indexOf(' TO ')) {
						var range = value_facet.split(' TO ');
						if (range[0][0] == '[') {
							range[0] = range[0].substring(1, range[0].length);
						}
						if (range[1][range[1].length - 1] == ']') {
							range[1] = range[1].substring(0, range[1].length - 1);
						}

						if(cvetta_lang == 'it' || cVettaParams.price_separator == ',')  {
							var low_range = numberWithCommas(range[0]);
							var upper_range = numberWithCommas(range[1]);
						} else {
							var low_range = range[0];
							var upper_range = range[1];
						}

						value_facet_display = low_range/*Math.round(range[0])*/ + ' - ' + upper_range/*Math.round(range[1])*/;
					}

					var facetLabel = cVettaFacetLabels[facet] ? cVettaFacetLabels[facet] : facet;
					bufferFacets += '<li><a href="#" class="facet_applied" data-facet-name="' + facet + '" data-facet-value="' + value_facet + '" ><span class="facet-close-left"><strong>' + facetLabel + '</strong> > ' + value_facet_display + '</span><span class="facet-close-right"></span></a></li>';

				};

			};
			bufferFacets += '</ul></li>';
		}
		bufferFacets += '</ul>';
		document.getElementById('cvettaFacetContainer').innerHTML = bufferFacets;

		// suggestions
		var suggestions = false;
		var suggestionContent = '';
		//console.log('suggestions:');
		//console.log(data.spellcheck.suggestions);
		if (data.spellcheck && data.spellcheck.suggestions) {
			suggestionContent += '<div class="cvettaSuggestion"><div class="cvettaSuggestionTitle">'+cVettaLabels["didYouMean"]+'</div>';
			for (var i in data.spellcheck.suggestions) {
				var item = data.spellcheck.suggestions[i];
				//console.log(item);
				if (item.suggestion) {
					suggestions = true;
					var suggestion_length = item.suggestion.length;
					//console.log(suggestion_length);
					for (k=0; k<suggestion_length; k++ ) {
						//console.log(item.suggestion[k]);
						suggestionContent += '<a href="#" class="facet_suggestion" data-facet-value="' + item.suggestion[k] + '" >' + item.suggestion[k] + '</a> ';
					}
				}
				if (item.collationQuery) {
					suggestions = true;
					var collationQuery = item.collationQuery.substring(0, item.collationQuery.length -1);
					suggestionContent += '<a href="#" class="facet_suggestion" data-facet-value="' + collationQuery + '" >' + collationQuery + '</a> ';
				}
				/*if (item.misspellingsAndCorrections) {
					for (var z in item.misspellingsAndCorrections) {
						console.log(item.misspellingsAndCorrections[z]);
						suggestionContent += '<a class="suggestion_facet" href="#" >' + item.misspellingsAndCorrections[z] + '</a> ';
					}
				}*/
			};
			suggestionContent += '</div>';
		}
		if (suggestions) {
			document.getElementById('cvettaDidYouMeanContainer').innerHTML = suggestionContent;
		}

		document.getElementById('cvettaResultList').innerHTML = bufferContent;

	}

	// carico i banner publicitari
	if (cVettaParams.advertise) {
		loadAdvertise();
	}

	runListeners();

    var event = new CustomEvent('cVettaGetResults', { 'detail': data });
    document.dispatchEvent(event);
    console.log('dispateched cVettaGetResults');

}

function advertise(data) {
	var cvettaResultList = document.getElementById('cvettaResultList');
	var cvettaResultContainer = document.getElementById('cvettaResultContainer');
	var cvettaResultToolbar = document.getElementById('cvettaResultToolbar');
	var cvetta = document.getElementById('cvetta');

	if( data.response.numFound > 0 ){

		var bufferContent = '';
		var docs_lenght = data.response.docs.length;
		for (var i = 0; i < docs_lenght; i++) {
			var item = data.response.docs[i];
			bufferContent += '<div class="cvettarow">' + data.response.docs[i].itt_advertise + '</div>';
		};

		var cvettaAdvertiseList = document.getElementById('cvettaAdvertiseList');
		cvettaAdvertiseList.innerHTML = bufferContent;
		cvettaResultList.style.height = cvettaResultContainer.clientHeight - cvettaResultToolbar.clientHeight - 77/*cvettaAdvertiseList.clientHeight*/ +'px';

		var copylinks = cvetta.getElementsByTagName('a');
		var copy = copylinks[copylinks.length-1];
		copy.style.marginBottom = 77/*cvettaAdvertiseList.clientHeight*/ +'px';
	} else {
		cvettaResultList.style.height = cvettaResultContainer.clientHeight - cvettaResultToolbar.clientHeight +'px';
		var copylinks = cvetta.getElementsByTagName('a');
		var copy = copylinks[copylinks.length-1];
		copy.style.marginBottom = '5px';
	}
}

function setupFacetSlider(facet, lower, upper, lowerStart, upperEnd, step) {
	var slider = document.getElementById(facet + '_slider');

	noUiSlider.create(slider, {
		start: [ lowerStart, upperEnd ], // Handle start position
		step: step, // Slider moves in increments of 'xxx'
		margin: step, // Handles must be more than 'xxx' apart
		connect: true, // Display a colored bar between the handles
		behaviour: 'tap-drag', // Move handle on tap, bar is draggable
		range: { // Slider can select 'xxx' to 'yyy'
			'min': lower,
			'max': upper
		}
	});

	var snapValues = [
		document.getElementById('lower_' + facet),
		document.getElementById('upper_' + facet)
	];

	var lower_facet_info = document.getElementById('lower_' + facet + '_info');
	var upper_facet_info = document.getElementById('upper_' + facet + '_info');
	var cvetta_lang = cVettaParams.lang.substring(0,2);

	if(cvetta_lang == 'it' || cVettaParams.price_separator == ',')  {
		var lower_range_value = numberWithCommas(lowerStart);
		var upper_range_value = numberWithCommas(upperEnd);
	} else {
		var lower_range_value = lowerStart;
		var upper_range_value = upperEnd;
	}

	lower_facet_info.innerHTML = lower_range_value;
	upper_facet_info.innerHTML = upper_range_value;

	slider.noUiSlider.on('update', function( values, handle ) {
		if(cvetta_lang == 'it' || cVettaParams.price_separator == ',')  {
			var lower_range_value = numberWithCommas(document.getElementById('lower_' + facet).value);
			var upper_range_value = numberWithCommas(document.getElementById('upper_' + facet).value);
		} else {
			var lower_range_value = document.getElementById('lower_' + facet).value;
			var upper_range_value = document.getElementById('upper_' + facet).value;
		}

		snapValues[handle].value = values[handle];
		lower_facet_info.innerHTML = lower_range_value;
		upper_facet_info.innerHTML = upper_range_value;
	});
	slider.noUiSlider.on('change', function( values, handle ) {
		var start = snapValues[0].value;
		var end = snapValues[1].value;

		if (parseFloat(start) == parseFloat(lower) && parseFloat(end) == parseFloat(upper)) {
			// sono ai margini dei valori, tolgo il filtro
			if (typeof facet_applied[facet] != 'undefined') {
				delete facet_applied[facet];
			}
		} else {
			facet_applied[facet] = ['['+start+'+TO+'+end+']'];
		}

		loadData(0);
	});
}

function countProperties (obj) {
    var count = 0;

    for (var property in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, property)) {
            count++;
        }
    }

    return count;
}

//(function( ){
document.addEventListener("DOMContentLoaded", function(event) { // non funziona con IE8
	var threshold_width = (cVettaParams.hasOwnProperty('threshold_width') ? cVettaParams.threshold_width : 992);
	var cvettaBox = document.createElement('div');
	cvettaBox.setAttribute("id", "cvetta");
	cvettaBox.style.display = "none";
	cvettaBox.innerHTML = '' +
	    '<div id="cvettarowContainer" class="cvettarow">'+
	        '<div id="cvettaSuggestContainer"></div>'+
	        '<div id="cvettaFacetContainer" class="cvettacol cvettasidebar"></div>'+
       		'<input type="checkbox" id="cvetta-offcanvas-nav-trigger" /><label for="cvetta-offcanvas-nav-trigger" id ="cvetta-offcanvas-nav-trigger-label" class="cvettaIcon"></label>'+
       		'<div id="cvetta-offcanvas-background"></div>'+
	        '<div id="cvettaResultContainer" class="cvettacol cvettacontent">'+
	       		'<div id="cvettaResultsCountMobile"></div>'+
	            '<div id="cvettaResultToolbar">'+
	                '<div id="cvettaViewOptions">'+
	                    '<button id="cvettaBtnGridview" type="button" class="cvettaIcon cvettaGridView active" aria-label="List View"><svg width="75px" height="62px" viewBox="0 0 75 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><rect x="0" y="0" width="34.1" height="27.9" rx="4.13333333"></rect><rect x="0" y="34.1" width="34.1" height="27.9" rx="4.13333333"></rect><rect x="40.3" y="0" width="34.1" height="27.9" rx="4.13333333"></rect><rect x="40.3" y="34.1" width="34.1" height="27.9" rx="4.13333333"></rect></g></g></svg></button>'+
						'<button id="cvettaBtnListview" type="button" class="cvettaIcon cvettaListView" aria-label="Grid View"><svg width="75px" height="62px" viewBox="0 0 75 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><rect x="0" y="0" width="18.6" height="16.5333333" rx="4.13333333"></rect><rect x="0" y="22.7333333" width="18.6" height="16.5333333" rx="4.13333333"></rect><rect x="0" y="45.4666667" width="18.6" height="16.5333333" rx="4.13333333"></rect><rect x="24.8" y="0" width="49.6" height="16.5333333" rx="4.13333333"></rect><rect x="24.8" y="22.7333333" width="49.6" height="16.5333333" rx="4.13333333"></rect><rect x="24.8" y="45.4666667" width="49.6" height="16.5333333" rx="4.13333333"></rect></g></g></svg></button>'+
	                '</div>'+
	                '<div id="cvettaResultsCount"></div>'+
	                '<div id="cvettaSortResults">'+
	                    '<button id="cvettaBtnSortDefault" type="button" class="cvettaIcon active" aria-label="Sort default"><svg width="62px" height="62px" viewBox="0 0 62 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><path d="M31,0 C48.1208272,0 62,13.8791728 62,31 C62,48.1208272 48.1208272,62 31,62 C13.8791728,62 0,48.1208272 0,31 C0,13.8791728 13.8791728,0 31,0 Z M31,4 C16.0883118,4 4,16.0883118 4,31 C4,45.9116882 16.0883118,58 31,58 C45.9116882,58 58,45.9116882 58,31 C58,16.0883118 45.9116882,4 31,4 Z" fill-rule="nonzero"></path><path d="M31,8 C43.7025492,8 54,18.2974508 54,31 C54,43.7025492 43.7025492,54 31,54 C18.2974508,54 8,43.7025492 8,31 C8,18.2974508 18.2974508,8 31,8 Z M31,12 C20.5065898,12 12,20.5065898 12,31 C12,41.4934102 20.5065898,50 31,50 C41.4934102,50 50,41.4934102 50,31 C50,20.5065898 41.4934102,12 31,12 Z" fill-rule="nonzero"></path><circle cx="31" cy="31" r="13"></circle></g></g></svg></button>'+
						'<button id="cvettaBtnSortPriceAsc" type="button" class="cvettaIcon" aria-label="Sort ASC"><svg width="62px" height="62px" viewBox="0 0 62 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><path d="M31,0 C48.1208272,0 62,13.8791728 62,31 C62,48.1208272 48.1208272,62 31,62 C13.8791728,62 0,48.1208272 0,31 C0,13.8791728 13.8791728,0 31,0 Z M31.0445104,12 C30.3323442,12 29.620178,12.2820976 29.1216617,12.7757685 L29.1216617,12.7757685 L16.8011869,24.9764919 C15.7329377,25.9638336 15.7329377,27.6564195 16.8011869,28.7142857 C17.7982196,29.7721519 19.5074184,29.7721519 20.5756677,28.7142857 L20.5756677,28.7142857 L28.2670623,21.0271248 L28.2670623,48.3200723 C28.2670623,49.801085 29.4777448,51 30.9732938,51 C32.4688427,51 33.6795252,49.801085 33.6795252,48.3200723 L33.6795252,48.3200723 L33.6795252,21.0271248 L41.4421365,28.7142857 C42.4391691,29.7721519 44.148368,29.7721519 45.2166172,28.7142857 C45.7151335,28.1500904 46,27.5153707 46,26.8101266 C46,26.1048825 45.7151335,25.3996383 45.2166172,24.9059675 L45.2166172,24.9059675 L32.8961424,12.7052441 C32.3976261,12.2115732 31.7566766,12 31.0445104,12 Z"></path></g></g></svg></button>'+
						'<button id="cvettaBtnSortPriceDesc" type="button" class="cvettaIcon" aria-label="Sort DESC"><svg width="62px" height="62px" viewBox="0 0 62 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><path d="M31,0 C48.1208272,0 62,13.8791728 62,31 C62,48.1208272 48.1208272,62 31,62 C13.8791728,62 0,48.1208272 0,31 C0,13.8791728 13.8791728,0 31,0 Z M31.0445104,12 C30.3323442,12 29.620178,12.2820976 29.1216617,12.7757685 L29.1216617,12.7757685 L16.8011869,24.9764919 C15.7329377,25.9638336 15.7329377,27.6564195 16.8011869,28.7142857 C17.7982196,29.7721519 19.5074184,29.7721519 20.5756677,28.7142857 L20.5756677,28.7142857 L28.2670623,21.0271248 L28.2670623,48.3200723 C28.2670623,49.801085 29.4777448,51 30.9732938,51 C32.4688427,51 33.6795252,49.801085 33.6795252,48.3200723 L33.6795252,48.3200723 L33.6795252,21.0271248 L41.4421365,28.7142857 C42.4391691,29.7721519 44.148368,29.7721519 45.2166172,28.7142857 C45.7151335,28.1500904 46,27.5153707 46,26.8101266 C46,26.1048825 45.7151335,25.3996383 45.2166172,24.9059675 L45.2166172,24.9059675 L32.8961424,12.7052441 C32.3976261,12.2115732 31.7566766,12 31.0445104,12 Z" transform="translate(31.000000, 31.000000) scale(1, -1) translate(-31.000000, -31.000000) "></path></g></g></svg></button>'+
	                '</div>'+
	            '</div>'+
	            '<div id="cvettaResultList" class="cvettarow cvettaGridView"></div>'+
	            '<div id="cvettaAdvertiseList" class="cvettarow"></div>'+
	            '<div id="cvettaDidYouMeanContainer"></div>'+
	        '</div>'+
	    '</div>';

	if(document.getElementById('cVetta-form') != null) {
		document.getElementById('cVetta-form').appendChild(cvettaBox);
	}

    var cvetta_search = document.getElementById("cvetta-search");
    if(cvetta_search != null) {
        // rimuovo i campi hidden della form
        var form = document.getElementById("cVetta-form");
        var hidden_inputs = form.getElementsByClassName('cVetta-hidden-input');
        //console.log("hidden_inputs:");
        //console.log(hidden_inputs);
        var hidden_inputs_lenght = hidden_inputs.length;
        //console.log('hidden_inputs_lenght:'+hidden_inputs_lenght);
        if (hidden_inputs_lenght > 0) {
            for (i = 0; i < hidden_inputs_lenght; i++) {
                //console.log("rimuovo: "+hidden_inputs[0]);
                //if (hidden_inputs[i] != undefined) {
                    form.removeChild(hidden_inputs[0]);
                //}
            }
        }
        // reimposto i valori in ricerca e i filtri hidden alla form
        var urlParams = cVettaGetUrlParams();
        var cvetta_search_text = urlParams.q ? urlParams.q : cVettaGetCookie('cvetta.search.text'); // riprendo il testo della ricerca dando prioritÃ  al parametro in request e poi al cookie
        document.cookie = "cvetta.search.text="+cvetta_search_text; // reimposto il nuovo cookie
        facet_applied = JSON.parse(cVettaGetUrlCookies(urlParams) || cVettaGetCookie('cvetta.search.facets') || '{}'); // riprendo i filtri settati in ordine: 1) request url; 2) layer ricerca
        document.cookie = "cvetta.search.facets="+JSON.stringify(facet_applied); // reimposto il nuovo cookie
        for (var i in facet_applied) {
            var values = facet_applied[i];
            if( typeof values === 'string' ) {
                values = [ values ];
            }
            for (var value = 0; value < values.length; value++) {
                //console.log("addHidden facets_label[]:"+i);
                //console.log("addHidden facets_value[]:"+values[value]);
                addHidden(form, 'facets_label[]', i);
                addHidden(form, 'facets_value[]', values[value]);
            }
        }
        if (cvetta_search_text) {
            cvetta_search.value = cvetta_search_text.replace(/\+/g, ' ');
        }
        cvetta_search.addEventListener("keyup", function (event) {
            //console.log(event.which);
            // le frecce non le considero
            if (event.which >= 37 && event.which <= 40) {
                return;
            }

            if (event.which == 27) {
                // ESC
                document.getElementById("cvetta").style.display = 'none';
                return;
            }

            //console.log( this.value.length );
            if (this.value.length > 2) {
                facet_applied = {}; // azzero i filtri
                loadData(0);

                if (window.innerWidth < threshold_width) {
                    var me = this;
                    clearTimeout(blurTimeout);
                    blurTimeout = setTimeout(function () {
                        me.blur();
                    }, 5000);
                }
            }
        });

        cvetta_search.addEventListener("click", function (event) {
            event.stopPropagation();
        });

        // intercettiamo il tasto destro del mouse
        cvetta_search.addEventListener("mousedown", function (event) {
            if (event.which == 3) {
                event.stopPropagation();
            }
        });

        cvetta_search.addEventListener("focus", function (event) {
            // nella versione mobile sposto il div di ricerca
            if (window.innerWidth < threshold_width) {
                var cvettaForm = document.getElementById('cVetta-form');
                if (cvettaForm.querySelector('#cvetta')) {
                    var cvettaFormSearch1 = document.getElementById('cvetta-form-search');
                    var cvettaFormSearch2 = cvettaFormSearch1;
                    var cvetta1 = document.getElementById('cvetta');
                    var cvetta2 = cvetta1;
                    cvettaFormSearch1.parentNode.removeChild(cvettaFormSearch1);
                    cvetta1.parentNode.removeChild(cvetta1);
                    document.body.appendChild(cvettaFormSearch2);
                    document.body.appendChild(cvetta2);
                    //cvetta_search.select();
                    //cvetta_search.focus(); // da problemi sullo smartphone, da browser va bene

                    var body = document.getElementsByTagName('body')[0];
                    body.className = body.className.replace(/(?:^|\s)cvetta-search-active(?!\S)/g, '');
                    body.className += ' cvetta-search-active';
                }
                setTimeout(function() {document.getElementById('cvetta-search').focus()}, 0);
            } else {
                this.select();
            }

            if (this.value.length > 2) {
                loadData(0);
                document.getElementById("cvetta").style.display = 'block';

                var body = document.getElementsByTagName('body')[0];
                body.className = body.className.replace(/(?:^|\s)cvetta-search-active(?!\S)/g, '');
                body.className += ' cvetta-search-active';

                if (window.innerWidth < threshold_width) {
                    var me = this;
                    clearTimeout(blurTimeout);
                    blurTimeout = setTimeout(function () {
                        me.blur();
                    }, 5000);
                }
            }
        });
    }

    if(document.getElementById("cvetta") != null) {
        document.getElementById("cvetta").addEventListener("click", function (event) {
            event.stopPropagation();
        });

        document.addEventListener("click", function (event) {
            hidecVetta();
        });

		if(cVettaParams.viewMode == 1) {
			listViewcVetta(document.getElementById("cvettaBtnListview"));
        }

        document.getElementById("cvettaBtnListview").addEventListener("click", function (event) {
            event.preventDefault();

            listViewcVetta(this);
			/*
            var cvettaBtnGridview = document.getElementById("cvettaBtnGridview");
            cvettaBtnGridview.className = cvettaBtnGridview.className.replace(/(?:^|\s)active(?!\S)/g, '');
            this.className = this.className.replace(/(?:^|\s)active(?!\S)/g, '');
            this.className += ' active';

            var cvettaResultList = document.getElementById("cvettaResultList");
            cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaGridView(?!\S)/g, '');
            cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaListView(?!\S)/g, '');
            cvettaResultList.className += ' cvettaListView'; */
        });

        document.getElementById("cvettaBtnGridview").addEventListener("click", function (event) {
            event.preventDefault();

            var cvettaBtnListview = document.getElementById("cvettaBtnListview");
            cvettaBtnListview.className = cvettaBtnListview.className.replace(/(?:^|\s)active(?!\S)/g, '');
            this.className = this.className.replace(/(?:^|\s)active(?!\S)/g, '');
            this.className += ' active';

            var cvettaResultList = document.getElementById("cvettaResultList");
            cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaListView(?!\S)/g, '');
            cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaGridView(?!\S)/g, '');
            cvettaResultList.className += ' cvettaGridView';
        });

        document.getElementById("cvettaBtnSortDefault").addEventListener("click", function (event) {
            event.preventDefault();

            var cvettaBtnSortPriceAsc = document.getElementById("cvettaBtnSortPriceAsc");
            cvettaBtnSortPriceAsc.className = cvettaBtnSortPriceAsc.className.replace(/(?:^|\s)active(?!\S)/g, '');
            var cvettaBtnSortPriceDesc = document.getElementById("cvettaBtnSortPriceDesc");
            cvettaBtnSortPriceDesc.className = cvettaBtnSortPriceDesc.className.replace(/(?:^|\s)active(?!\S)/g, '');
            this.className += ' active';

            // sort by score
            sort = 'score';
            direction = 'desc';
            loadData(0);
        });

        document.getElementById("cvettaBtnSortPriceAsc").addEventListener("click", function (event) {
            event.preventDefault();

            var cvettaBtnSortDefault = document.getElementById("cvettaBtnSortDefault");
            cvettaBtnSortDefault.className = cvettaBtnSortDefault.className.replace(/(?:^|\s)active(?!\S)/g, '');
            var cvettaBtnSortPriceDesc = document.getElementById("cvettaBtnSortPriceDesc");
            cvettaBtnSortPriceDesc.className = cvettaBtnSortPriceDesc.className.replace(/(?:^|\s)active(?!\S)/g, '');
            this.className += ' active';

            // sort by price asc
            sort = 'price';
            direction = 'asc';
            loadData(0);
        });

        document.getElementById("cvettaBtnSortPriceDesc").addEventListener("click", function (event) {
            event.preventDefault();

            var cvettaBtnSortDefault = document.getElementById("cvettaBtnSortDefault");
            cvettaBtnSortDefault.className = cvettaBtnSortDefault.className.replace(/(?:^|\s)active(?!\S)/g, '');
            var cvettaBtnSortPriceAsc = document.getElementById("cvettaBtnSortPriceAsc");
            cvettaBtnSortPriceAsc.className = cvettaBtnSortPriceAsc.className.replace(/(?:^|\s)active(?!\S)/g, '');
            this.className += ' active';

            // sort by price desc
            sort = 'price';
            direction = 'desc';
            loadData(0);
        });

        // questo serve per chiudere il menÃ¹ laterale in modalitÃ  mobile
        document.getElementById("cvetta-offcanvas-background").addEventListener("click", function (event) {
            event.preventDefault();

            document.getElementById('cvetta-offcanvas-nav-trigger').checked = false;
        });

        document.getElementById("cvettaResultList").addEventListener("scroll", function (event) {
            //console.log(this.scrollTop);
            if (this.scrollHeight - this.scrollTop - this.offsetHeight < 50) {
                if (results_loaded < total_products && canScroll) {
                    var new_start = parseInt(start) + parseInt(rows)
                    //console.log('carico ' + new_start);
                    event.stopPropagation();
                    loadData(parseInt(start) + parseInt(rows));
                }
                /*else {
                 console.log('fine raggiunta '+ results_loaded + ' di ' + total_products);
                 }*/
            }
        });

        document.getElementById("cvetta-search-close").addEventListener("click", function (event) {
            event.preventDefault();

            hidecVetta();

            //document.getElementById("cvetta-search").value = '';
        });

        // copyright
        var copyright = document.createElement('a');
        copyright.setAttribute("style", "position:absolute !important;bottom:0px !important;right:20px !important;z-index:9999999 !important;background-color:#ffffff !important;display:block !important;text-indent:0px !important;visibility:visible !important;font-size:80% !important;padding:2px !important;text-decoration:none !important; color:#000000 !important;");
        copyright.setAttribute("href", "http://www.cvetta.it/");
        copyright.setAttribute("target", "_blank");
        if (window.innerWidth > (threshold_width - 1))
			copyright.innerHTML = 'powered by ';
        copyright.innerHTML += '<svg width="12" height="14" viewBox="0 0 80 93" xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg">' +
            '<g>' +
            '<circle class="fill" r="5.099609" cy="23.6" cx="31.089063" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"/>' +
            '<circle class="fill" r="5.099609" cy="23.6" cx="48.689062" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"/>' +
            '<path class="fill" d="m45.989075,90c-1.299988,2 -3.5,3.399994 -6.100006,3.399994c-2.600006,0 -4.799988,-1.399994 -6.100006,-3.399994c-1.299988,2 -3.5,3.399994 -6.099976,3.399994c-4,0 -7.200012,-3.199982 -7.200012,-7.199982c0,-2.100006 0.799988,-4 2.200012,-5.300018c2.5,-2.199982 7,-4.100006 10.799988,-7.799988c4.399994,-4.200012 6.299988,-7.600006 6.299988,-7.600006s2,3.399994 6.300018,7.600006c3.799988,3.699982 8.299988,5.5 10.799988,7.799988c1.399994,1.300018 2.200012,3.200012 2.200012,5.300018c0,4 -3.200012,7.199982 -7.200012,7.199982c-2.399994,0 -4.700012,-1.399994 -5.899994,-3.399994z" fill="#385F89"/>' +
            '<path class="fill" d="m55.189056,67.200012c0,7.200012 4.899994,10.399994 11.100006,10.399994c7,0 9.199982,-3.799988 9.199982,-5.399994c0,-1.399994 -1.099976,-2.899994 -2.899994,-2.899994c-2.199982,0 -3.299988,2.799988 -6,2.799988c-2.699982,0 -3.600006,-2.5 -3.600006,-5.100006c0,-2.099976 0.700012,-4.799988 3.900024,-4.799988c2.799988,0 3.599976,2.799988 5.699982,2.799988c1.600006,0 2.800018,-1.099976 2.800018,-2.899994c0,-2.699982 -3.600006,-5.399994 -9.100006,-5.399994c-6.200012,0.099976 -11.100006,3.299988 -11.100006,10.5zm-15.299988,-37.900024c0,0 -0.600006,3.299988 -2.700012,5.600006c-2.099976,2.399994 -5.199982,3.899994 -5.199982,3.899994s3,1.5 4.600006,2.799988c1.599976,1.300018 3.299988,3.700012 3.299988,3.700012s1.600006,-2.399994 3.299988,-3.700012c1.600006,-1.299988 4.600006,-2.799988 4.600006,-2.799988s-3.100006,-1.5 -5.199982,-3.899994c-2.100006,-2.399994 -2.700012,-5.600006 -2.700012,-5.600006zm23.100006,9.800018c2.399994,1.200012 4.600006,2.899994 6.200012,4.899994c5.5,6.5 16.199982,29.899994 7.5,36.300018c-8.700012,6.399994 -27.600006,-6.100006 -34.100006,-13.900024c-1,-1.199982 -1.899994,-2.5 -2.600006,-3.799988c-0.700012,1.299988 -1.600006,2.600006 -2.600006,3.799988c-6.5,7.800018 -25.399994,20.300018 -34.099991,13.900024c-8.699997,-6.400024 2,-29.800018 7.499985,-36.300018c1.600006,-1.899994 3.800018,-3.600006 6.200012,-4.899994c-5.200012,-2.799988 -7.899994,-8.700012 -7.899994,-13.600006c0,-15 15.799988,-25.5 15.799988,-25.5c0.700012,7.800018 7.300018,13.899994 15.200012,13.899994c8,0 14.5,-6.099976 15.199982,-13.899994c0,0 15.800018,10.5 15.800018,25.5c-0.200012,4.899994 -2.900024,10.799988 -8.100006,13.600006zm-48.200012,-15.100006c0,-6.100006 5,-11.100006 11.100006,-11.100006c6.100006,0 11.100006,5 11.100006,11.100006c0,6.100006 -5,11.100006 -11.100006,11.100006c-6.200012,0 -11.100006,-5 -11.100006,-11.100006zm28,0c0,-6.100006 5,-11.100006 11.100006,-11.100006c6.100006,0 11.100006,5 11.100006,11.100006c0,6.100006 -5,11.100006 -11.100006,11.100006c-6.100006,0 -11.100006,-5 -11.100006,-11.100006z" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"/>' +
            '</g>' +
            '</svg>';
        document.getElementById('cvetta').appendChild(copyright);
    }
});
//})();

function hidecVetta() {
    var threshold_width = (cVettaParams.hasOwnProperty('threshold_width') ? cVettaParams.threshold_width : 992);

    document.getElementById("cvetta").style.display = 'none';

    if (window.innerWidth < threshold_width) {
        var cvettaForm = document.getElementById('cVetta-form');
        if (!cvettaForm.querySelector('#cvetta')) {
            var cvettaFormSearch1 = document.getElementById('cvetta-form-search');
            var cvettaFormSearch2 = cvettaFormSearch1;
            var cvetta1 = document.getElementById('cvetta');
            var cvetta2 = cvetta1;
            cvettaFormSearch1.parentNode.removeChild(cvettaFormSearch1);
            cvetta1.parentNode.removeChild(cvetta1);
            document.getElementById('cVetta-form').appendChild(cvettaFormSearch2);
            document.getElementById('cVetta-form').appendChild(cvetta2);
        }
    }

    var body = document.getElementsByTagName('body')[0];
    body.className = body.className.replace(/(?:^|\s)cvetta-search-active(?!\S)/g, '');
}

// dentro questa funzione mettere gli ascoltatori soltanto degli elementi che vengono distrutti e ricreati ogni volta dalla chiamata ajax
// gli altri metterli nell'evento DOMContentLoaded
function runListeners() {
    var a_facet = document.querySelectorAll("a.facet_field");
    for (var i = 0; i < a_facet.length; i++) {
        a_facet[i].removeEventListener("click", facetFieldClick);
        a_facet[i].addEventListener("click", facetFieldClick);
    }

    var a_facet = document.querySelectorAll("a.facet_suggestion");
    for (var i = 0; i < a_facet.length; i++) {
        a_facet[i].removeEventListener("click", facetSuggestionClick);
        a_facet[i].addEventListener("click", facetSuggestionClick);
    }

    var a_facet = document.querySelectorAll("a.facet_applied");
    for (var i = 0; i < a_facet.length; i++) {
        a_facet[i].removeEventListener("click", facetAppliedClick);
        a_facet[i].addEventListener("click", facetAppliedClick);
    }

    var a_clear_filters = document.getElementById("cvetta-clear-filters");
    if (a_clear_filters) {
        a_clear_filters.removeEventListener("click", clearFiltersClick);
        a_clear_filters.addEventListener("click", clearFiltersClick);
    }

    var cvetta_facet = document.querySelectorAll(".cvetta-facet");
    for (var i = 0; i < cvetta_facet.length; i++) {
        cvetta_facet[i].removeEventListener("click", cvettaFacetClick);
        cvetta_facet[i].addEventListener("click", cvettaFacetClick);
    }

    // click sul view more delle faccette
    var a_view_more_subact = document.querySelectorAll("a.view-more-subact");
    for (var i = 0; i < a_view_more_subact.length; i++) {
        a_view_more_subact[i].removeEventListener("click", viewMoreSubactClick);
        a_view_more_subact[i].addEventListener("click", viewMoreSubactClick);
    }

    // click sul prodotto
    var a_item_detail = document.querySelectorAll("a.item-detail");
    for (var i = 0; i < a_item_detail.length; i++) {
        a_item_detail[i].removeEventListener("click", itemDetailClick);
        a_item_detail[i].addEventListener("click", itemDetailClick);
    }

    // tasto destro del mouse
    var a_item_detail = document.querySelectorAll("a.item-detail");
    for (var i = 0; i < a_item_detail.length; i++) {
        a_item_detail[i].removeEventListener("click", itemDetailClick);
        a_item_detail[i].addEventListener("click", itemDetailClick);

        a_item_detail[i].removeEventListener("mousedown", function(event) {
            if (event.which == 3)
                itemDetailClick
        });
        a_item_detail[i].addEventListener("mousedown", itemDetailClick);
    }
}

function facetFieldClick(event) {
    event.preventDefault();
    //console.log(facet_applied);
    var facet_name = this.getAttribute('data-facet-name');
    var facet_value = this.getAttribute('data-facet-value');
    if (facet_applied[facet_name]) {
        facet_applied[facet_name].push(facet_value)
    } else {
        facet_applied[facet_name] = [facet_value];
    };
    loadData(0);
}

function facetSuggestionClick(event) {
    event.preventDefault();

    var facet_value = this.getAttribute('data-facet-value');
    //console.log('OKK3 ' + facet_value);
    document.getElementById("cvetta-search").value = facet_value;
    facet_applied = {}; // azzero i filtri precedenti
    loadData(0);
}

function facetAppliedClick(event) {
    event.preventDefault();
    var facet_name = this.getAttribute('data-facet-name');
    var facet_value = this.getAttribute('data-facet-value');

	/*if (facet_value[0] == '[' && facet_value[facet_value.length -1] == ']' && facet_value.indexOf(' TO ')) {
	 var range = facet_value.split(' - ');
	 if (range[0][0] == '[') {
	 range[0] = range[0].substring(1, range[0].length);
	 }
	 if (range[1][range[1].length - 1] == ']') {
	 range[1] = range[1].substring(0, range[1].length - 1);
	 }
	 facet_value = '[' + range[0] + ' TO ' + range[1] + ']';
	 }*/

    for (var i in facet_applied[facet_name]) {
        if (facet_applied[facet_name][i] == facet_value) {
            facet_applied[facet_name].splice(i, 1);
            break;
        }
    }
    loadData(0);
}

function clearFiltersClick (event) {
    event.preventDefault();
    facet_applied = {};
    loadData(0);
}

function cvettaFacetClick (event) {
    event.preventDefault();

    if (this.className.match(/(?:^|\s)cvetta-plus(?!\S)/)) {
        this.className = this.className.replace(/(?:^|\s)cvetta-plus(?!\S)/g, '');
        this.className += ' cvetta-minus';
    } else {
        this.className = this.className.replace(/(?:^|\s)cvetta-minus(?!\S)/g, '');
        this.className += ' cvetta-plus';
    }
}

function viewMoreSubactClick (event) {
    event.preventDefault();

    var parentUl = this.parentNode.parentNode;

    if (this.className.match(/(?:^|\s)view-more-subact-full(?!\S)/)) {
        this.className = this.className.replace(/(?:^|\s)view-more-subact-full(?!\S)/g, '');
        parentUl.className = parentUl.className.replace(/(?:^|\s)subfull(?!\S)/g, '');
    } else {
        this.className += ' view-more-subact-full';
        parentUl.className += ' subfull';
    }

    this.setAttribute('title', this.getAttribute('title') == cVettaLabels["showMore"] ? cVettaLabels["showLess"] : cVettaLabels["showMore"]);
}

function itemDetailClick (event) {
    event.preventDefault();

    var sku = encodeURIComponent(this.getAttribute('data-item-sku'));
    var desc = encodeURIComponent(this.getAttribute('data-item-desc'));
    var cvetta_search_text = document.getElementById("cvetta-search").value;
    if (cvetta_search_text.length < 3) {
        cvetta_search_text = cVettaGetCookie('cvetta.search.text');
    }
    var urlParams = cVettaGetUrlParams();
    if (urlParams.q) {
        cvetta_search_text = urlParams.q;
    }
    if (cvetta_search_text.length < 3) {
        document.getElementById("cvetta-search").focus();
        console.log("campo Cerca non compilato");
        return false;
    }
    var word = encodeURIComponent(cvetta_search_text);
    var url = this.getAttribute('data-item-url');
    var url = encodeURIComponent(url);
    var url_destination = "//ws10b.cvetta.io/click.php?prodid="+sku+"&desc="+desc+"&idvetrina="+cVettaParams.idvetrina+"&word="+word+"&furl="+url;

    location.href = url_destination;
}

function appendGA (url) {
    var word = encodeURIComponent(document.getElementById("cvetta-search").value);

    // aggiungiamo gli eventuali parametri che tracciano su GA il click dell'utente
    if(typeof cVettaGAParams !== "undefined") {
        if(cVettaGAParams.utm_source != '' || cVettaGAParams.utm_medium != '' || cVettaGAParams.utm_campaign != '')
            url += '?utm_term='+word;
        if (cVettaGAParams.utm_source != '')
            url += "&utm_source=" + cVettaGAParams.utm_source;
        if (cVettaGAParams.utm_medium != '')
            url += '&utm_medium=' + cVettaGAParams.utm_medium;
        if (cVettaGAParams.utm_campaign != '')
            url += "&utm_campaign=" + cVettaGAParams.utm_campaign;
    }

    return url;
}

// simil printf
if (!String.prototype.format) {
    String.prototype.format = function() {
        var args = arguments;
        return this.replace(/{(\d+)}/g, function(match, number) {
            return typeof args[number] != 'undefined' ? args[number] : match;
        });
    };
}


function templateProduct(data, item) {
    var bufferContent = '<div class="cvettaProduct cvettacol">';
    var url = appendGA(item.url_key);
    var cvetta_lang = cVettaParams.lang.substring(0,2);

    bufferContent += '<a class="cvettarow item-detail" href="' + url + '" data-item-sku="' + htmlEscape(item.sku) + '" data-item-desc="' + htmlEscape(item.name) + '" data-item-url="' + url + '" >';
    
    if( item.image && item.image.match(/[^/]+(jpg|jpeg|png|gif)$/) !== null ) {
        bufferContent += '<div class="cvettacol cvettaProductImage"><img src="' + item.image + '" alt="' + htmlEscape(item.name) + '" /></div>';
    }else{
        bufferContent += '<div class="cvettacol cvettaProductImage"><img src="/media/catalog/product/cache/1/small_image/260x250/9df78eab33525d08d6e5fb8d27136e95/images/catalog/product/placeholder/small_image.jpg" /></div>';
    }
    bufferContent += '<div class="cvettacol cvettaProductTitle">' + data.highlighting[item.sku].name;
	/*if (data.highlighting[item.sku].description) {
	 bufferContent += '<div class="cvettaProductDescription">' + data.highlighting[item.sku].description + '</div>';
	 }*/
    bufferContent += '</div>';
    bufferContent += '<div class="cvettarow"><div class="cvettacol cvettaProductPrice">';
    if(item.price) {
        if (isNumeric(item.price)) {
            item.price = parseFloat(item.price).toFixed(2);
        }
        if (isNumeric(item.full_price)) {
            item.full_price = parseFloat(item.full_price).toFixed(2);
        }
        if (isNumeric(item.special_price)) {
            if (parseFloat(item.special_price) < parseFloat(item.price) && !item.full_price) {
                item.full_price = item.price;
                item.price = parseFloat(item.special_price).toFixed(2);
            }
        }

        if(cvetta_lang == 'it' || cVettaParams.price_separator == ',')  {
            var full_price_format = numberWithCommas(item.full_price);
            var price_format = numberWithCommas(item.price);
        } else {
            var full_price_format = item.full_price;
            var price_format = item.price;
        }

        if (item.full_price && isNumeric(item.full_price) && parseFloat(item.price) < parseFloat(item.full_price)) {
            bufferContent += '<span class="cvettaProductFullPrice">' + full_price_format + ' ' + cVettaParams.currency + '</span>';
        }

        var price = isNumeric(item.price) ? price_format + ' ' + cVettaParams.currency : price_format;
        bufferContent += '<span class="cvettaProductRegularPrice">'+price+'</span>';
    } else {
        if (cVettaParams.noprice_text) {
            bufferContent += cVettaParams.noprice_text;
        } else {
            bufferContent += '<span class="cvettaProductSKU">STAR Ref. '+item.sku+'</span>';
        }
    }

    bufferContent += '</div></div>';
    bufferContent += '</a>';
    bufferContent += '</div>';

    return bufferContent;
}

function numberWithCommas(x) {
    if(x)
        return x.toString().replace(/\./g, ',');
    else
        return x;
}

function isNumeric(n) {
    return !isNaN(parseFloat(n)) && isFinite(n);
}

function listViewcVetta(element) {
    var cvettaBtnGridview = document.getElementById("cvettaBtnGridview");
    cvettaBtnGridview.className = cvettaBtnGridview.className.replace(/(?:^|\s)active(?!\S)/g, '');
    element.className = element.className.replace(/(?:^|\s)active(?!\S)/g, '');
    element.className += ' active';

    var cvettaResultList = document.getElementById("cvettaResultList");
    cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaGridView(?!\S)/g, '');
    cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaListView(?!\S)/g, '');
    cvettaResultList.className += ' cvettaListView';
}

function cVettaGetCookie(cname) {
    var name = cname + "=";
    var ca = document.cookie.split(';');
    for(var i = 0; i <ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0)==' ') {
            c = c.substring(1);
        }
        if (c.indexOf(name) == 0) {
            return c.substring(name.length,c.length);
        }
    }
    return "";
}

function addHidden(theForm, key, value) {
    // Create a hidden input element, and append it to the form:
    var input = document.createElement('input');
    var uniquid = guid();
    input.type = 'hidden';
    input.id = 'cvetta-hidden-facet-'+uniquid;
    input.name = key;
    input.value = value;
    input.className = 'cVetta-hidden-input';
    theForm.appendChild(input);
}

function guid() {
  function s4() {
    return Math.floor((1 + Math.random()) * 0x10000)
      .toString(16)
      .substring(1);
  }
  return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
    s4() + '-' + s4() + s4() + s4();
}

function cVettaGetUrlParams() {
  var url = document.location.href;
  //var url = decodeURI(document.location.href);
  //var url = decodeURIComponent(document.location.href);

  // get query string from url (optional) or window
  var queryString = url ? url.split('?')[1] : window.location.search.slice(1);

  // we'll store the parameters here
  var obj = {};

  // if query string exists
  if (queryString) {

    // stuff after # is not part of query string, so get rid of it
    queryString = queryString.split('#')[0];

    // split our query string into its component parts
    var arr = queryString.split('&');

    for (var i=0; i<arr.length; i++) {
      // separate the keys and the values
      var a = arr[i].split('=');

      // in case params look like: list[]=thing1&list[]=thing2
      var paramNum = undefined;
      var paramName = a[0].replace(/\[\d*\]/, function(v) {
        paramNum = v.slice(1,-1);
        return '';
      });

      // set parameter value (use 'true' if empty)
      var paramValue = typeof(a[1])==='undefined' ? true : a[1];

      // (optional) keep case consistent
      paramName = paramName.toLowerCase();
      paramValue = paramValue.toLowerCase();

      // if parameter name already exists
      if (obj[paramName]) {
        // convert value to array (if still string)
        if (typeof obj[paramName] === 'string') {
          obj[paramName] = [obj[paramName]];
        }
        // if no array index number specified...
        if (typeof paramNum === 'undefined') {
          // put the value on the end of the array
          obj[paramName].push(paramValue);
        }
        // if array index number specified...
        else {
          // put the value at that index number
          obj[paramName][paramNum] = paramValue;
        }
      }
      // if param name doesn't exist yet, set it
      else {
        obj[paramName] = paramValue;
      }
    }
  }

  return obj;
}

function cVettaGetUrlCookies(params) {
    var facets_label = [];
    var facets_value = [];
    for (var key in params) {
        if (params.hasOwnProperty(key)) {
            //console.log(key + " -> " + params[key]);

            if (decodeURIComponent(key) == 'facets_label' || decodeURIComponent(key) == 'facets_label[]') {
                if (typeof params[key] === 'string') {
                    facets_label.push(params[key]);
                } else {
                    facets_label = params[key];
                }
            }
            if (decodeURIComponent(key) == 'facets_value' || decodeURIComponent(key) == 'facets_value[]') {
                if (typeof params[key] === 'string') {
                    facets_value.push(params[key]);
                } else {
                    facets_value = params[key];
                }
            }
        }
    }

    var facets_label_lenght = facets_label.length;
    if (facets_label_lenght > 0) {
        var cookies = {};
        for (var i = 0; i < facets_label_lenght; i++) {
            var label = decodeURIComponent(facets_label[i]); //.replace(/\+/g, ' ');
            var value = decodeURIComponent(facets_value[i]).replace(/\+to\+/g, '+TO+');
            //console.log(label+'|'+value);

            cookies[label] = cookies[label] || [];
            //cookies[facets_label[i]] = cookies[facets_label[i]] || [];
            cookies[label].push(value);
            //cookies[facets_label[i]].push(facets_value[i]);
        }
        //console.log(cookies);
        return JSON.stringify(cookies);
    }

    return false;
}

function htmlEscape(str) {
    return str
        .replace(/&/g, '&amp;')
        .replace(/"/g, '&quot;')
        .replace(/'/g, '&#39;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;');
}